#!/bin/bash

# -----------------------------------------------------------------------------
# Axon Ivy Engine systemd service installer
#
# Copyright 1994 - 2019 Axon Ivy AG
# -----------------------------------------------------------------------------

MODE=$1 # supports: [non-interactive]

USER=`whoami`
if [ "$USER" != "root" ]
then
	echo "You must be root to execute this script!"
	exit 0
fi

GREEN='\033[0;32m'
RED='\033[0;31m'
NC='\033[0m'



# greet
if [ "$MODE" != "non-interactive" ]
then
	echo ""
	printf "Welcome to the ${GREEN}Axon Ivy${NC} Engine systemd service installer!\n"
	echo ""
fi



# select engine to use
INSTALLDIR=`pwd` # current path
INSTALLDIR=`readlink -m $INSTALLDIR/..` # normalized parent path!
if [ "$MODE" != "non-interactive" ]
then
	echo "---"
	echo "Please set your Axon Ivy Engine installation directory"
	read -p "Select directory: [$INSTALLDIR"] SELECTEDDIR
	if [ -z $SELECTEDDIR ]
	then
		echo ""
	else
		INSTALLDIR=$SELECTEDDIR
		echo ""
	fi
fi



# determine user and group
REAL_USER=`ls -ld $INSTALLDIR | awk '{print $3}'`
REAL_GROUP=`ls -ld $INSTALLDIR | awk '{print $4}'`
if [ "$MODE" != "non-interactive" ]
then
	echo "---"
	echo "Please enter the user under which the Axon Ivy Engine Service should run (must NOT be root)"
	read -p "Select user: [$REAL_USER"] SELECTEDUSER
	if [ -z $SELECTEDUSER ]
	then
		echo ""
	else
		REAL_USER=$SELECTEDUSER
		echo ""
	fi

	echo "---"
	echo "Please enter the group under which the Axon Ivy Engine Service should run"
	read -p "Select group: [$REAL_GROUP"] SELECTEDGROUP
	if [ -z $SELECTEDGROUP ]
	then
		echo ""
	else
		REAL_GROUP=$SELECTEDGROUP
		echo ""
	fi

fi



# name of the service
## support versioned/named instances so that 'axonivy-engine-7' and 'axonivy-engine-8' can run on same host
## but fallback to 'AxonIvyEngine' if other non default installation dir was used.
SERVICENAME=${INSTALLDIR##*/}
if [ "${SERVICENAME#axonivy-engine-}" = "$SERVICENAME" ]
then
    SERVICENAME=AxonIvyEngine
fi
if [ "$MODE" != "non-interactive" ]
then
    echo "---"
	echo "Please enter the name under which the Axon Ivy Engine Service should be made known"
	read -p "Select service name: [$SERVICENAME"] SELECTEDNAME
	if [ -z $SELECTEDNAME ]
	then
		echo ""
	else
		SERVICENAME=$SELECTEDNAME
		echo ""
	fi
fi



# install
echo "Installing Axon Ivy Engine Service..."
if [ -d "/etc/systemd/system" ]
then
	SERVICE="/lib/systemd/system/$SERVICENAME.service"
    echo "- Copying start script to '$SERVICE'"
    sed "s|/<installationdir>|$INSTALLDIR|g; s|<user>|$REAL_USER|g; s|<group>|$REAL_GROUP|g" AxonIvyEngine.service > $SERVICE
	printf "...${GREEN}done${NC}\n"
	echo "- Reloading daemons"
	systemctl daemon-reload
	printf "...${GREEN}done${NC}\n"
	echo ""
	echo "$SERVICENAME.service was successfully installed"

    echo "starting $SERVICENAME.service"
    systemctl start ${SERVICENAME}.service &
	printf "To start the engine automatically during boot, execute '${GREEN}systemctl enable $SERVICENAME.service${NC}'"

else
	printf "The directory '/etc/systemd/system' does not exist, ${RED}canceling${NC} installation. Does your Linux distribution support systemd?\n"
	exit 1
fi

echo ""

exit 0

