-- ---------------------------------------------------------------------------------
-- 
-- Converts the ivy system database from version 67 to version 68
-- 
-- ---------------------------------------------------------------------------------
-- 
-- This script was automatically generated. Do not edit it. Instead edit the source file
-- 
-- ---------------------------------------------------------------------------------
-- Database: Microsoft SQL Server
-- ---------------------------------------------------------------------------------
-- Copyright:
-- Axon Ivy AG, Baarerstrasse 12, 6300 Zug
-- ---------------------------------------------------------------------------------

-- Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_TaskQuery
GO

DROP VIEW IWA_CaseQuery
GO

DROP VIEW IWA_ProcessStart
GO


-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_ApplicationDeleteTrigger
GO

-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_UserDeleteTrigger
GO

-- Create new added tables
CREATE TABLE IWA_SecurityMember
(
  SecurityMemberId VARCHAR(210) NOT NULL,
  Name VARCHAR(200) NOT NULL,
  MemberName VARCHAR(201) NOT NULL,
  DisplayName VARCHAR(200) NULL,
  ApplicationId BIGINT NOT NULL,
  Enabled BIT NOT NULL DEFAULT 0,
  "Type" INTEGER NOT NULL,
  PRIMARY KEY (SecurityMemberId)
)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_SecurityMember') AND name='IWA_SecurityMemberNameIdx')
  CREATE INDEX IWA_SecurityMemberNameIdx ON IWA_SecurityMember (Name)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_SecurityMember') AND name='IWA_SecurityMemberMemberNameIdx')
  CREATE INDEX IWA_SecurityMemberMemberNameIdx ON IWA_SecurityMember (MemberName)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_SecurityMember') AND name='IWA_SecurityMemberDisplayNameIdx')
  CREATE INDEX IWA_SecurityMemberDisplayNameIdx ON IWA_SecurityMember (DisplayName)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_SecurityMember') AND name='IWA_SecurityMemberEnabledIdx')
  CREATE INDEX IWA_SecurityMemberEnabledIdx ON IWA_SecurityMember (Enabled)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_SecurityMember') AND name='IWA_SecurityMemberTypeIdx')
  CREATE INDEX IWA_SecurityMemberTypeIdx ON IWA_SecurityMember ("Type")
GO


-- Drop unique constraint which depend on changed columns or was deleted
-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_Role'
GO

-- Added columns of table IWA_Role
ALTER TABLE IWA_Role ADD
  SecurityMemberId VARCHAR(210) NULL
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_Role ADD UNIQUE (ApplicationId, Name)
GO

-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_User'
GO

-- Added columns of table IWA_User
ALTER TABLE IWA_User ADD
  SecurityMemberId VARCHAR(210) NULL
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_User ADD UNIQUE (ApplicationId, Name)
GO

-- Added columns of table IWA_Case
ALTER TABLE IWA_Case ADD
  CreatorId VARCHAR(210) NULL
GO
ALTER TABLE IWA_Case ADD
  OwnerId VARCHAR(210) NULL
GO

-- Added columns of table IWA_Task
ALTER TABLE IWA_Task ADD
  WorkerId VARCHAR(210) NULL
GO
ALTER TABLE IWA_Task ADD
  ActivatorId VARCHAR(210) NULL
GO
ALTER TABLE IWA_Task ADD
  OriginalActivatorId VARCHAR(210) NULL
GO
ALTER TABLE IWA_Task ADD
  ExpiryActivatorId VARCHAR(210) NULL
GO

-- Added columns of table IWA_SignalEvent
ALTER TABLE IWA_SignalEvent ADD SentById VARCHAR(210) NULL;
GO

-- Migrate Priority
EXEC sp_rename 'IWA_Task.Priority', 'OriginalPriority', 'COLUMN';
GO
ALTER TABLE IWA_Task ADD Priority INTEGER NULL;
GO
UPDATE IWA_Task SET Priority = CASE WHEN IsExpired = 1 THEN ExpiryPriority ELSE OriginalPriority END;
GO
ALTER TABLE IWA_Task ALTER COLUMN Priority INTEGER NOT NULL;
GO

-- Migrate IWA_User
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT CONCAT('USER-', UserId), Name, FullName, CONCAT('#', Name), ApplicationId, 1, 1 FROM IWA_User;
GO
UPDATE IWA_User SET SecurityMemberId = CONCAT('USER-', UserId);
GO
ALTER TABLE IWA_User ALTER COLUMN SecurityMemberId VARCHAR(210) NOT NULL;
GO
ALTER TABLE IWA_User ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

-- Migrate IWA_Role
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT CONCAT('ROLE-', RoleId), Name, DisplayNameTemplate, Name, ApplicationId, 1, 0 FROM IWA_Role;
GO
UPDATE IWA_Role SET SecurityMemberId = CONCAT('ROLE-', RoleId);
GO
ALTER TABLE IWA_Role ALTER COLUMN SecurityMemberId VARCHAR(210) NOT NULL;
GO
ALTER TABLE IWA_Role ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

-- Migrate IWA_Case.CreatorId
UPDATE IWA_Case SET CreatorId = CASE
   WHEN CreatorUserId IS NOT NULL THEN CONCAT('USER-', CreatorUserId)
   WHEN CreatorUserName IS NOT NULL THEN CONCAT('USER-', ApplicationId, '-', CreatorUserName)
   ELSE NULL
  END;
GO
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT DISTINCT
    CreatorId,
    CASE WHEN CreatorUserName IS NOT NULL THEN CreatorUserName ELSE CreatorId END,
    CASE WHEN CreatorUserName IS NOT NULL THEN CreatorUserName ELSE CreatorId END,
    CONCAT('#', CASE WHEN CreatorUserName IS NOT NULL THEN CreatorUserName ELSE CreatorId END),
    ApplicationId,
    0,
    1
  FROM IWA_Case
  WHERE IWA_Case.CreatorId IS NOT NULL AND
        IWA_Case.CreatorId NOT IN (SELECT SecurityMemberId FROM IWA_SecurityMember);
GO
ALTER TABLE IWA_Case ADD FOREIGN KEY (CreatorId) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

-- Migrate IWA_Case.OwnerId
UPDATE IWA_Case SET OwnerName = NULL WHERE OwnerName = '#';

UPDATE IWA_Case SET OwnerId = CASE
   WHEN OwnerRoleId IS NOT NULL THEN CONCAT('ROLE-', OwnerRoleId)
   WHEN OwnerUserId IS NOT NULL THEN CONCAT('USER-', OwnerUserId)
   WHEN OwnerName LIKE '#%' THEN CONCAT('USER-', ApplicationId, '-', SUBSTRING(OwnerName, 2, 200))
   WHEN OwnerName IS NOT NULL THEN CONCAT('ROLE-', ApplicationId, '-', OwnerName)
   ELSE NULL
  END;
GO
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT DISTINCT
    OwnerId,
    CASE
      WHEN OwnerName LIKE '#%' THEN SUBSTRING(OwnerName, 2, 200)
      WHEN OwnerName IS NOT NULL THEN OwnerName
      ELSE OwnerId
    END,
    CASE
      WHEN OwnerName LIKE '#%' THEN SUBSTRING(OwnerName, 2, 200)
      WHEN OwnerName IS NOT NULL THEN OwnerName
      ELSE OwnerId
    END,
    CASE
      WHEN OwnerName IS NOT NULL THEN OwnerName
      WHEN OwnerId LIKE 'USER-%' THEN CONCAT('#', OwnerId)
      ELSE OwnerId
    END,
    ApplicationId,
    0,
    CASE WHEN OwnerId LIKE 'USER-%' THEN 1 ELSE 0 END
  FROM IWA_Case
  WHERE IWA_Case.OwnerId IS NOT NULL AND
        IWA_Case.OwnerId NOT IN (SELECT SecurityMemberId FROM IWA_SecurityMember);
GO
ALTER TABLE IWA_Case ADD FOREIGN KEY (OwnerId) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

-- Migrate IWA_Task.WorkerId
UPDATE IWA_Task SET WorkerId = CASE
   WHEN WorkerUserId IS NOT NULL THEN CONCAT('USER-', WorkerUserId)
   WHEN WorkerUserName IS NOT NULL THEN CONCAT('USER-', ApplicationId, '-', WorkerUserName)
   ELSE NULL
  END;
GO
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT DISTINCT
    WorkerId,
    CASE WHEN WorkerUserName IS NOT NULL THEN WorkerUserName ELSE WorkerId END,
    CASE WHEN WorkerUserName IS NOT NULL THEN WorkerUserName ELSE WorkerId END,
    CONCAT('#', CASE WHEN WorkerUserName IS NOT NULL THEN WorkerUserName ELSE WorkerId END),
    ApplicationId,
    0,
    1
  FROM IWA_Task
  WHERE IWA_Task.WorkerId IS NOT NULL AND
        IWA_Task.WorkerId NOT IN (SELECT SecurityMemberId FROM IWA_SecurityMember);
GO
ALTER TABLE IWA_Task ADD FOREIGN KEY (WorkerId) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

-- Migrate IWA_Task.OriginalActivatorId
UPDATE IWA_Task SET ActivatorName = NULL WHERE ActivatorName = '#';

UPDATE IWA_Task SET OriginalActivatorId =
  CASE
   WHEN ActivatorUserId IS NOT NULL THEN CONCAT('USER-', ActivatorUserId)
   WHEN ActivatorRoleId IS NOT NULL THEN CONCAT('ROLE-', ActivatorRoleId)
   WHEN ActivatorName LIKE '#%' THEN CONCAT('USER-', ApplicationId, '-', SUBSTRING(ActivatorName, 2, 200))
   WHEN ActivatorName IS NOT NULL THEN CONCAT('ROLE-', ApplicationId, '-', ActivatorName)
   ELSE NULL
  END;
GO
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT DISTINCT
    OriginalActivatorId,
    CASE
      WHEN ActivatorName LIKE '#%' THEN SUBSTRING(ActivatorName, 2, 200)
      WHEN ActivatorName IS NOT NULL THEN ActivatorName
      ELSE OriginalActivatorId
    END,
    CASE
      WHEN ActivatorName LIKE '#%' THEN SUBSTRING(ActivatorName, 2, 200)
      WHEN ActivatorName IS NOT NULL THEN ActivatorName
      ELSE OriginalActivatorId
    END,
    CASE
      WHEN ActivatorName IS NOT NULL THEN ActivatorName
      WHEN OriginalActivatorId LIKE 'USER-%' THEN CONCAT('#', OriginalActivatorId)
      ELSE OriginalActivatorId
    END,
    ApplicationId,
    0,
    CASE WHEN OriginalActivatorId LIKE 'USER-%' THEN 1 ELSE 0 END
  FROM IWA_Task
  WHERE IWA_Task.OriginalActivatorId IS NOT NULL AND
        IWA_Task.OriginalActivatorId NOT IN (SELECT SecurityMemberId FROM IWA_SecurityMember);
ALTER TABLE IWA_Task ADD FOREIGN KEY (OriginalActivatorId) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

-- Migrate IWA_Task.ExpiryActivatorId
UPDATE IWA_Task SET ExpiryActivatorName = NULL WHERE ExpiryActivatorName = '#';

UPDATE IWA_Task SET ExpiryActivatorId =
  CASE
   WHEN ExpiryActivatorUserId IS NOT NULL THEN CONCAT('USER-', ExpiryActivatorUserId)
   WHEN ExpiryActivatorRoleId IS NOT NULL THEN CONCAT('ROLE-', ExpiryActivatorRoleId)
   WHEN ExpiryActivatorName LIKE '#%' THEN CONCAT('USER-', ApplicationId, '-', SUBSTRING(ExpiryActivatorName, 2, 200))
   WHEN ExpiryActivatorName IS NOT NULL THEN CONCAT('ROLE-', ApplicationId, '-', ExpiryActivatorName)
   ELSE NULL
  END;
GO
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT DISTINCT
    ExpiryActivatorId,
    CASE
      WHEN ExpiryActivatorName LIKE '#%' THEN SUBSTRING(ExpiryActivatorName, 2, 200)
      WHEN ExpiryActivatorName IS NOT NULL THEN ExpiryActivatorName
      ELSE ExpiryActivatorId
    END,
    CASE
      WHEN ExpiryActivatorName LIKE '#%' THEN SUBSTRING(ExpiryActivatorName, 2, 200)
      WHEN ExpiryActivatorName IS NOT NULL THEN ExpiryActivatorName
      ELSE ExpiryActivatorId
    END,
    CASE
      WHEN ExpiryActivatorName IS NOT NULL THEN ExpiryActivatorName
      WHEN ExpiryActivatorId LIKE 'USER-%' THEN CONCAT('#', ExpiryActivatorId)
      ELSE ExpiryActivatorId
    END,
    ApplicationId,
    0,
    CASE WHEN ExpiryActivatorId LIKE 'USER-%' THEN 1 ELSE 0 END
  FROM IWA_Task
  WHERE IWA_Task.ExpiryActivatorId IS NOT NULL AND
        IWA_Task.ExpiryActivatorId NOT IN (SELECT SecurityMemberId FROM IWA_SecurityMember);
GO
ALTER TABLE IWA_Task ADD FOREIGN KEY (ExpiryActivatorId) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

-- Migrate IWA_Task.ActivatorId (CurrentActivator)
UPDATE IWA_Task SET ActivatorId = CASE WHEN IsExpired = 1 THEN ExpiryActivatorId ELSE OriginalActivatorId END;
GO
ALTER TABLE IWA_Task ADD FOREIGN KEY (ActivatorId) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

-- Migrate StartElement
ALTER TABLE IWA_StartElement ADD ActivatorId VARCHAR(210) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO
UPDATE IWA_StartElement SET ActivatorId =
  CASE
   WHEN ActivatorRoleId IS NOT NULL THEN CONCAT('ROLE-', ActivatorRoleId)
   WHEN ActivatorUserId IS NOT NULL THEN CONCAT('USER-', ActivatorUserId)
   ELSE NULL
  END;
GO

-- Migrate StartEventElement
ALTER TABLE IWA_StartEventElement ADD ActivatorId VARCHAR(210) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO
UPDATE IWA_StartEventElement SET ActivatorId =
  CASE
   WHEN ActivatorRoleId IS NOT NULL THEN CONCAT('ROLE-', ActivatorRoleId)
   ELSE NULL
  END;
GO

-- Migrate WebServiceProcStartElement
ALTER TABLE IWA_WebServiceProcStartElement ADD ActivatorId VARCHAR(210) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO
UPDATE IWA_WebServiceProcStartElement SET ActivatorId =
  CASE
   WHEN ActivatorRoleId IS NOT NULL THEN CONCAT('ROLE-', ActivatorRoleId)
   ELSE NULL
  END;
GO

-- Migrate IWA_WorkflowEvent.UserId
DROP INDEX IWA_WorkflowEvent_UserIdIndex ON IWA_WorkflowEvent;
GO
EXEC sp_rename 'IWA_WorkflowEvent.UserId', 'WorkerUserId', 'COLUMN';
GO
ALTER TABLE IWA_WorkflowEvent ADD UserId VARCHAR(210);
GO
UPDATE IWA_WorkflowEvent SET UserId = CASE
   WHEN WorkerUserId IS NOT NULL THEN CONCAT('USER-', WorkerUserId)
   WHEN UserName IS NOT NULL AND UserName NOT LIKE ('Unknown User (Session %') THEN CONCAT('USER-', ApplicationId, '-', UserName)
   ELSE NULL
  END;
GO
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT DISTINCT
    UserId,
    CASE WHEN UserName IS NOT NULL THEN UserName ELSE UserId END,
    CASE WHEN UserName IS NOT NULL THEN UserName ELSE UserId END,
    CONCAT('#', CASE WHEN UserName IS NOT NULL THEN UserName ELSE UserId END),
    ApplicationId,
    0,
    1
  FROM IWA_WorkflowEvent
  WHERE IWA_WorkflowEvent.UserId IS NOT NULL AND
        IWA_WorkflowEvent.UserId NOT IN (SELECT SecurityMemberId FROM IWA_SecurityMember);
GO
ALTER TABLE IWA_WorkflowEvent ADD FOREIGN KEY (UserId) REFERENCES IWA_SecurityMember(SecurityMemberId);

-- Migrate IWA_SignalEvent.SentById
UPDATE IWA_SignalEvent SET SentById = CASE
   WHEN SentByUserId IS NOT NULL THEN CONCAT('USER-', SentByUserId)
   WHEN SentByUserName IS NOT NULL THEN CONCAT('USER-', ApplicationId, '-', SentByUserName)
   ELSE NULL
  END;
GO
INSERT INTO IWA_SecurityMember (SecurityMemberId, Name, DisplayName, MemberName, ApplicationId, Enabled, Type)
  SELECT DISTINCT
    SentById,
    CASE WHEN SentByUserName IS NOT NULL THEN SentByUserName ELSE SentById END,
    CASE WHEN SentByUserName IS NOT NULL THEN SentByUserName ELSE SentById END,
    CONCAT('#', CASE WHEN SentByUserName IS NOT NULL THEN SentByUserName ELSE SentById END),
    ApplicationId,
    0,
    1
  FROM IWA_SignalEvent
  WHERE IWA_SignalEvent.SentById IS NOT NULL AND
        IWA_SignalEvent.SentById NOT IN (SELECT SecurityMemberId FROM IWA_SecurityMember);
GO

ALTER TABLE IWA_SignalEvent ADD FOREIGN KEY (SentById) REFERENCES IWA_SecurityMember(SecurityMemberId);
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_SignalEvent') AND name='IWA_SignalEvent_SentById')
  CREATE INDEX IWA_SignalEvent_SentById ON IWA_SignalEvent (SentById)
GO

-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_ApplicationDeleteTrigger
  ON IWA_Application FOR DELETE AS

  DELETE IWA_WebService FROM IWA_WebService, deleted
  WHERE IWA_WebService.ApplicationId = deleted.ApplicationId

  DELETE IWA_SignalEvent FROM IWA_SignalEvent, deleted
  WHERE IWA_SignalEvent.ApplicationId = deleted.ApplicationId

  DELETE IWA_ApplicationProperty FROM IWA_ApplicationProperty, deleted
  WHERE IWA_ApplicationProperty.ApplicationId = deleted.ApplicationId

  DELETE IWA_RestServiceClient FROM IWA_RestServiceClient, deleted
  WHERE IWA_RestServiceClient.ApplicationId = deleted.ApplicationId

  DELETE IWA_ContentManagementSystem FROM IWA_ContentManagementSystem, deleted
  WHERE IWA_ContentManagementSystem.ApplicationId = deleted.ApplicationId

  DELETE IWA_User FROM IWA_User, deleted
  WHERE IWA_User.ApplicationId = deleted.ApplicationId

  DELETE IWA_ProcessModel FROM IWA_ProcessModel, deleted
  WHERE IWA_ProcessModel.ApplicationId = deleted.ApplicationId

  DELETE IWA_Environment FROM IWA_Environment, deleted
  WHERE IWA_Environment.ApplicationId = deleted.ApplicationId

  DELETE IWA_ExternalDatabase FROM IWA_ExternalDatabase, deleted
  WHERE IWA_ExternalDatabase.ApplicationId = deleted.ApplicationId

  DELETE IWA_Role FROM IWA_Role, deleted
  WHERE IWA_Role.ApplicationId = deleted.ApplicationId

  DELETE IWA_SecurityMember FROM IWA_SecurityMember, deleted
  WHERE IWA_SecurityMember.ApplicationId = deleted.ApplicationId

  DELETE IWA_GlobalVariable FROM IWA_GlobalVariable, deleted
  WHERE IWA_GlobalVariable.ApplicationId = deleted.ApplicationId

  DELETE IWA_ExternalNamingAndDirectory FROM IWA_ExternalNamingAndDirectory, deleted
  WHERE IWA_ExternalNamingAndDirectory.ApplicationId = deleted.ApplicationId

  DELETE IWA_SecurityDescriptor FROM IWA_SecurityDescriptor, deleted
  WHERE IWA_SecurityDescriptor.SecurityDescriptorId = deleted.SecurityDescriptorId


GO


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_UserDeleteTrigger
  ON IWA_User FOR DELETE AS

  UPDATE IWA_StartElement  SET IWA_StartElement.ActivatorUserId=NULL
  FROM IWA_StartElement, deleted
  WHERE IWA_StartElement.ActivatorUserId = deleted.UserId

  UPDATE IWA_Case  SET IWA_Case.CreatorUserId=NULL
  FROM IWA_Case, deleted
  WHERE IWA_Case.CreatorUserId = deleted.UserId

  UPDATE IWA_Case  SET IWA_Case.OwnerUserId=NULL
  FROM IWA_Case, deleted
  WHERE IWA_Case.OwnerUserId = deleted.UserId

  UPDATE IWA_Task  SET IWA_Task.WorkerUserId=NULL
  FROM IWA_Task, deleted
  WHERE IWA_Task.WorkerUserId = deleted.UserId

  UPDATE IWA_Task  SET IWA_Task.ActivatorUserId=NULL
  FROM IWA_Task, deleted
  WHERE IWA_Task.ActivatorUserId = deleted.UserId

  UPDATE IWA_Task  SET IWA_Task.ExpiryActivatorUserId=NULL
  FROM IWA_Task, deleted
  WHERE IWA_Task.ExpiryActivatorUserId = deleted.UserId

  UPDATE IWA_WorkflowEvent  SET IWA_WorkflowEvent.WorkerUserId=NULL
  FROM IWA_WorkflowEvent, deleted
  WHERE IWA_WorkflowEvent.WorkerUserId = deleted.UserId

  UPDATE IWA_Note  SET IWA_Note.UserId=NULL
  FROM IWA_Note, deleted
  WHERE IWA_Note.UserId = deleted.UserId

  UPDATE IWA_SignalEvent  SET IWA_SignalEvent.SentByUserId=NULL
  FROM IWA_SignalEvent, deleted
  WHERE IWA_SignalEvent.SentByUserId = deleted.UserId

  UPDATE IWA_PageArchive  SET IWA_PageArchive.UserId=NULL
  FROM IWA_PageArchive, deleted
  WHERE IWA_PageArchive.UserId = deleted.UserId

  UPDATE IWA_SecurityDescriptor  SET IWA_SecurityDescriptor.OwnerUserId=NULL
  FROM IWA_SecurityDescriptor, deleted
  WHERE IWA_SecurityDescriptor.OwnerUserId = deleted.UserId

  DELETE IWA_UserRole FROM IWA_UserRole, deleted
  WHERE IWA_UserRole.UserId = deleted.UserId

  DELETE IWA_UserAbsence FROM IWA_UserAbsence, deleted
  WHERE IWA_UserAbsence.UserId = deleted.UserId

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.UserId = deleted.UserId

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.SubstituteUserId = deleted.UserId

  DELETE IWA_UserProperty FROM IWA_UserProperty, deleted
  WHERE IWA_UserProperty.UserId = deleted.UserId

  DELETE IWA_AccessControl FROM IWA_AccessControl, deleted
  WHERE IWA_AccessControl.UserId = deleted.UserId

  DELETE IWA_UserLocation FROM IWA_UserLocation, deleted
  WHERE IWA_UserLocation.UserId = deleted.UserId


GO


-- Create new indexes of table IWA_Role
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Role') AND name='IWA_Role_SecurityMemberIdIndex')
  CREATE INDEX IWA_Role_SecurityMemberIdIndex ON IWA_Role (SecurityMemberId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_User') AND name='IWA_User_SecurityMemberIdIndex')
  CREATE INDEX IWA_User_SecurityMemberIdIndex ON IWA_User (SecurityMemberId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_StartElement') AND name='IWA_StartElement_ActivatorIdIndex')
  CREATE INDEX IWA_StartElement_ActivatorIdIndex ON IWA_StartElement (ActivatorId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_StartEventElement') AND name='IWA_StartEventElement_ActivatorIdIndex')
  CREATE INDEX IWA_StartEventElement_ActivatorIdIndex ON IWA_StartEventElement (ActivatorId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_WebServiceProcStartElement') AND name='IWA_WebServiceProcStartElement_ActivatorIdIndex')
  CREATE INDEX IWA_WebServiceProcStartElement_ActivatorIdIndex ON IWA_WebServiceProcStartElement (ActivatorId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Case') AND name='IWA_Case_CreatorIdIndex')
  CREATE INDEX IWA_Case_CreatorIdIndex ON IWA_Case (CreatorId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Case') AND name='IWA_Case_OwnerIdIndex')
  CREATE INDEX IWA_Case_OwnerIdIndex ON IWA_Case (OwnerId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Task') AND name='IWA_Task_WorkerIdIndex')
  CREATE INDEX IWA_Task_WorkerIdIndex ON IWA_Task (WorkerId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Task') AND name='IWA_Task_ActivatorIdIndex')
  CREATE INDEX IWA_Task_ActivatorIdIndex ON IWA_Task (ActivatorId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Task') AND name='IWA_Task_OriginalActivatorIdIndex')
  CREATE INDEX IWA_Task_OriginalActivatorIdIndex ON IWA_Task (OriginalActivatorId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Task') AND name='IWA_Task_ExpiryActivatorIdIndex')
  CREATE INDEX IWA_Task_ExpiryActivatorIdIndex ON IWA_Task (ExpiryActivatorId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_WorkflowEvent') AND name='IWA_WorkflowEvent_UserIdIndex')
  CREATE INDEX IWA_WorkflowEvent_UserIdIndex ON IWA_WorkflowEvent (UserId)
GO


-- Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_TaskQuery
(
  TaskId,
  CaseId,
  BusinessCaseId,
  ProcessModelId,
  ApplicationId,
  StartTaskSwitchEventId,
  EndTaskSwitchEventId,
  TaskStartId,
  TaskEndId,
  WorkerId,
  WorkerUserId,
  WorkerUserName,
  WorkerUserDisplayName,
  WorkerSessionId,
  ActivatorId,
  OriginalActivatorId,
  ActivatorRoleId,
  ActivatorUserId,
  ActivatorName,
  ActivatorDisplayName,
  ActivatorAvailable,
  ExpiryActivatorId,
  ExpiryActivatorRoleId,
  ExpiryActivatorUserId,
  ExpiryActivatorName,
  ExpiryActivatorDisplayName,
  ExpiryPriority,
  ExpiryTimestamp,
  ExpiryTaskStartElementPid,
  IsExpired,
  ExpiredCreatorTaskId,
  TimeoutedCreatorIntrmdtEventId,
  DelayTimestamp,
  "State",
  RequestPath,
  DisplayNameTemplate,
  Name,
  DisplayDescriptionTemplate,
  Description,
  Priority,
  OriginalPriority,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  FailedTimeoutTimestamp,
  NumberOfFailures,
  NumberOfResumes,
  Category,
  IsUpdatedOnStart,
  IsOffline,
  CurrentActivatorName,
  CurrentActivatorDisplayName,
  CurrentActivatorRoleId,
  CurrentActivatorUserId
)
AS
  SELECT
    IWA_Task.TaskId,
    IWA_Task.CaseId,
    IWA_Task.BusinessCaseId,
    IWA_Task.ProcessModelId,
    IWA_Task.ApplicationId,
    IWA_Task.StartTaskSwitchEventId,
    IWA_Task.EndTaskSwitchEventId,
    IWA_Task.TaskStartId,
    IWA_Task.TaskEndId,
    IWA_Task.WorkerId,
    IWA_Task.WorkerUserId,
    WorkerUser.Name,
    WorkerUser.DisplayName,
    IWA_Task.WorkerSessionId,
    IWA_Task.ActivatorId,
    IWA_Task.OriginalActivatorId,
    IWA_Task.ActivatorRoleId,
    IWA_Task.ActivatorUserId,
    OriginalActivator.MemberName,
    OriginalActivator.DisplayName,
    CurrentActivator.Enabled,
    IWA_Task.ExpiryActivatorId,
    IWA_Task.ExpiryActivatorRoleId,
    IWA_Task.ExpiryActivatorUserId,
    ExpiryActivator.MemberName,
    ExpiryActivator.DisplayName,
    IWA_Task.ExpiryPriority,
    IWA_Task.ExpiryTimestamp,
    IWA_Task.ExpiryTaskStartElementPid,
    IWA_Task.IsExpired,
    IWA_Task.ExpiredCreatorTaskId,
    IWA_Task.TimeoutedCreatorIntrmdtEventId,
    IWA_Task.DelayTimestamp,
    IWA_Task."State",
    IWA_Task.RequestPath,
    IWA_Task.DisplayNameTemplate,
    IWA_Task.Name,
    IWA_Task.DisplayDescriptionTemplate,
    IWA_Task.Description,
    IWA_Task.Priority,
    IWA_Task.OriginalPriority,
    IWA_Task.StartTimestamp,
    IWA_Task.EndTimestamp,
    IWA_Task.BusinessCalendar,
    IWA_Task.WorkingTime,
    IWA_Task.BusinessRuntime,
    IWA_Task.FailedTimeoutTimestamp,
    IWA_Task.NumberOfFailures,
    IWA_Task.NumberOfResumes,
    IWA_Task.Category,
    IWA_Task.IsUpdatedOnStart,
    IWA_Task.IsOffline,
    CurrentActivator.MemberName,
    CurrentActivator.DisplayName,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryActivatorRoleId WHEN 0 THEN IWA_Task.ActivatorRoleId END,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryActivatorUserId WHEN 0 THEN IWA_Task.ActivatorUserId END
  FROM IWA_Task
    LEFT OUTER JOIN IWA_SecurityMember AS CurrentActivator ON IWA_Task.ActivatorId = CurrentActivator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS OriginalActivator ON IWA_Task.OriginalActivatorId = OriginalActivator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS ExpiryActivator ON IWA_Task.ExpiryActivatorId = ExpiryActivator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS WorkerUser ON IWA_Task.WorkerId = WorkerUser.SecurityMemberId
GO

CREATE VIEW IWA_CaseQuery
(
  CaseId,
  BusinessCaseId,
  Kind,
  ApplicationId,
  ProcessModelId,
  TaskStartId,
  CreatorId,
  CreatorUserId,
  CreatorUserName,
  CreatorUserDisplayName,
  CreatorTaskId,
  EnvironmentId,
  DisplayNameTemplate,
  Name,
  DisplayDescriptionTemplate,
  Description,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  "State",
  Priority,
  Stage,
  OwnerId,
  OwnerRoleId,
  OwnerUserId,
  OwnerName,
  Category
)
AS
  SELECT
    IWA_Case.CaseId,
    IWA_Case.BusinessCaseId,
    IWA_Case.Kind,
    IWA_Case.ApplicationId,
    IWA_Case.ProcessModelId,
    IWA_Case.TaskStartId,
    IWA_Case.CreatorId,
    IWA_Case.CreatorUserId,
    Creator.Name,
    Creator.DisplayName,
    IWA_Case.CreatorTaskId,
    IWA_Case.EnvironmentId,
    IWA_Case.DisplayNameTemplate,
    IWA_Case.Name,
    IWA_Case.DisplayDescriptionTemplate,
    IWA_Case.Description,
    IWA_Case.StartTimestamp,
    IWA_Case.EndTimestamp,
    IWA_Case.BusinessCalendar,
    IWA_Case.WorkingTime,
    IWA_Case.BusinessRuntime,
    IWA_Case."State",
    IWA_Case.Priority,
    IWA_Case.Stage,
    IWA_Case.OwnerId,
    IWA_Case.OwnerRoleId,
    IWA_Case.OwnerUserId,
    Owner.MemberName,
    IWA_Case.Category
  FROM IWA_Case
    LEFT OUTER JOIN IWA_SecurityMember AS Creator ON IWA_Case.CreatorId = Creator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS Owner ON IWA_Case.OwnerId = Owner.SecurityMemberId
GO

CREATE VIEW IWA_ProcessStart
(
  TaskElementId,
  ProcessModelVersionId,
  Name,
  Description,
  ProcessElementId,
  TaskStartId,
  StartRequestPath,
  UserFriendlyStartRequestPath,
  ActivatorId
)
AS
  SELECT
    IWA_TaskElement.TaskElementId,
    IWA_TaskElement.ProcessModelVersionId,
    IWA_TaskElement.Name,
    IWA_TaskElement.Description,
    IWA_TaskElement.ProcessElementId,
    IWA_TaskStart.TaskStartId,
    IWA_TaskStart.StartRequestPath,
    IWA_TaskStart.UserFriendlyStartRequestPath,
    IWA_StartElement.ActivatorId
  FROM IWA_TaskElement,
    IWA_TaskStart,
    IWA_StartElement
  WHERE IWA_TaskElement.TaskElementId = IWA_TaskStart.TaskElementId AND IWA_TaskStart.TaskStartId = IWA_StartElement.TaskStartId
UNION ALL
  SELECT
    IWA_TaskElement.TaskElementId,
    IWA_TaskElement.ProcessModelVersionId,
    IWA_TaskElement.Name,
    IWA_TaskElement.Description,
    IWA_TaskElement.ProcessElementId,
    IWA_TaskStart.TaskStartId,
    IWA_TaskStart.StartRequestPath,
    IWA_TaskStart.UserFriendlyStartRequestPath,
    IWA_StartEventElement.ActivatorId
  FROM IWA_TaskElement,
    IWA_TaskStart,
    IWA_StartEventElement
  WHERE IWA_TaskElement.TaskElementId = IWA_TaskStart.TaskElementId AND IWA_TaskStart.TaskStartId = IWA_StartEventElement.TaskStartId
UNION ALL
  SELECT
    IWA_TaskElement.TaskElementId,
    IWA_TaskElement.ProcessModelVersionId,
    IWA_TaskElement.Name,
    IWA_TaskElement.Description,
    IWA_TaskElement.ProcessElementId,
    IWA_TaskStart.TaskStartId,
    IWA_TaskStart.StartRequestPath,
    IWA_TaskStart.UserFriendlyStartRequestPath,
    IWA_WebServiceProcStartElement.ActivatorId
  FROM IWA_TaskElement,
    IWA_TaskStart,
    IWA_WebServiceProcStartElement
  WHERE IWA_TaskElement.TaskElementId = IWA_TaskStart.TaskElementId AND IWA_TaskStart.TaskStartId = IWA_WebServiceProcStartElement.TaskStartId
GO

CREATE VIEW IWA_SignalEventQuery
(
  SignalEventId,
  ApplicationId,
  SignalCode,
  SentTimestamp,
  SentById,
  SentByUserName,
  SentByUserId,
  SentByTaskId,
  SentByProcessElementPid
)
AS
  SELECT
    IWA_SignalEvent.SignalEventId,
    IWA_SignalEvent.ApplicationId,
    IWA_SignalEvent.SignalCode,
    IWA_SignalEvent.SentTimestamp,
    IWA_SignalEvent.SentById,
    IWA_SecurityMember.Name,
    IWA_User.UserId,
    IWA_SignalEvent.SentByTaskId,
    IWA_SignalEvent.SentByProcessElementPid
  FROM IWA_SignalEvent
    LEFT OUTER JOIN IWA_SecurityMember ON IWA_SignalEvent.SentById = IWA_SecurityMember.SecurityMemberId
    LEFT OUTER JOIN IWA_User ON IWA_SecurityMember.SecurityMemberId = IWA_User.SecurityMemberId
GO


-- Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_Unique
GO

-- 
-- Update Version
-- 
UPDATE IWA_Version SET Version=68
GO
