# SQL script to convert the database for MySQL from version 78 to version 79


# Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_ApplicationDeleteTrigger;

# Drop trigger that has been deleted
DROP TRIGGER IWA_EnvironmentDeleteTrigger;
# Store Procedure to drop a foreign key constraint
CREATE PROCEDURE IWA_Drop_ForeignKey_Constraint(fk_schema VARCHAR(64), fk_table VARCHAR(64), fk_column VARCHAR(64))
BEGIN
  WHILE EXISTS(
    SELECT * FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
    WHERE REFERENCED_COLUMN_NAME IS NOT NULL
      AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
      AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
      AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
  ) 
  DO
    BEGIN
      SET @sqlstmt = (
        SELECT CONCAT('ALTER TABLE `',TABLE_SCHEMA,'`.',TABLE_NAME,' DROP FOREIGN KEY ',CONSTRAINT_NAME)
        FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
        WHERE REFERENCED_COLUMN_NAME IS NOT NULL
          AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
          AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
          AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
        LIMIT 1
      );
      PREPARE stmt1 FROM @sqlstmt;
      EXECUTE stmt1;
    END;
  END WHILE;
END;
;

CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_ExternalDatabase', 'ParentExternalDatabaseId');

# Drop trigger of no longer existing table
DROP TRIGGER IWA_ExternalDatabaseDeleteTrigger;

# Drop no longer existing table
DROP TABLE IWA_ExternalDatabaseProperty;

# Drop no longer existing table
DROP TABLE IWA_ExternalDatabase;

# Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_ApplicationDeleteTrigger
AFTER DELETE ON IWA_Application
FOR EACH ROW
BEGIN
  DELETE FROM IWA_Environment
  WHERE IWA_Environment.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_SignalEvent
  WHERE IWA_SignalEvent.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_Role
  WHERE IWA_Role.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_ApplicationProperty
  WHERE IWA_ApplicationProperty.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_RestServiceClient
  WHERE IWA_RestServiceClient.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_User
  WHERE IWA_User.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_ProcessModel
  WHERE IWA_ProcessModel.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_SecurityMember
  WHERE IWA_SecurityMember.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_SecurityDescriptor
  WHERE IWA_SecurityDescriptor.SecurityDescriptorId = OLD.SecurityDescriptorId;

END
;

# Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_ForeignKey_Constraint;

# Update Version
UPDATE IWA_Version SET Version=79;
