-- SQL script to convert the database for Microsoft SQL Server from version 80 to version 81


-- Drop unique constraint which depend on changed columns or was deleted
-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_TaskElement'
GO

-- Dropped columns of table IWA_TaskElement
ALTER TABLE IWA_TaskElement DROP COLUMN Name
GO

ALTER TABLE IWA_TaskElement DROP COLUMN Description
GO


-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_TaskElement ADD UNIQUE (ProcessModelVersionId, ProcessElementId, Kind)
GO

-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_TaskStart'
GO

-- Dropped columns of table IWA_TaskStart
ALTER TABLE IWA_TaskStart DROP COLUMN UserFriendlyStartRequestPath
GO


-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_TaskStart ADD UNIQUE (TaskElementId, StartRequestPath)
GO

-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_TaskEnd'
GO
select null
GO

-- Dropped columns of table IWA_TaskEnd
ALTER TABLE IWA_TaskEnd DROP COLUMN JoinRequestPath
GO

ALTER TABLE IWA_TaskEnd DROP COLUMN UserFriendlyJoinRequestPath
GO


-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_TaskEnd ADD UNIQUE (TaskElementId, JoinPathId)
GO

-- Create new unique constraint of table IWA_TaskElement
-- Skipping generation of constraint 'UNIQUE IWA_TaskElement_ProcessModelVersionIdProcessElementIdKind (ProcessModelVersionId, ProcessElementId, Kind)'. It was already re-generated in this script.

-- Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_Unique
GO

-- Update Version
UPDATE IWA_Version SET Version=81
GO
