# SQL script to convert the database for MySQL from version 81 to version 82

# Store Procedure to drop a foreign key constraint
CREATE PROCEDURE IWA_Drop_ForeignKey_Constraint(fk_schema VARCHAR(64), fk_table VARCHAR(64), fk_column VARCHAR(64))
BEGIN
  WHILE EXISTS(
    SELECT * FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
    WHERE REFERENCED_COLUMN_NAME IS NOT NULL
      AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
      AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
      AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
  ) 
  DO
    BEGIN
      SET @sqlstmt = (
        SELECT CONCAT('ALTER TABLE `',TABLE_SCHEMA,'`.',TABLE_NAME,' DROP FOREIGN KEY ',CONSTRAINT_NAME)
        FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
        WHERE REFERENCED_COLUMN_NAME IS NOT NULL
          AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
          AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
          AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
        LIMIT 1
      );
      PREPARE stmt1 FROM @sqlstmt;
      EXECUTE stmt1;
    END;
  END WHILE;
END;
;

CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_WebServiceFeature', 'WebServiceEnvironmentId');
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_WebServiceProperty', 'WebServiceEnvironmentId');
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_WebServiceEndpoint', 'WebServiceEnvironmentId');
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_WebServiceEndpoint', 'WebServicePortTypeId');
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_WebServiceEnvironment', 'WebServiceId');
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_WebServicePortType', 'WebServiceId');

# Drop no longer existing table
DROP TABLE IWA_WebServiceFeature;

# Drop no longer existing table
DROP TABLE IWA_WebServiceProperty;

# Drop no longer existing table
DROP TABLE IWA_WebServiceEndpoint;

# Drop no longer existing table
DROP TABLE IWA_WebServiceEnvironment;

# Drop no longer existing table
DROP TABLE IWA_WebServicePortType;

# Drop no longer existing table
DROP TABLE IWA_WebService;
# Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_ForeignKey_Constraint;

# Update Version
UPDATE IWA_Version SET Version=82;
