-- SQL script to convert the database for Microsoft SQL Server from version 83 to version 84

-- Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_CaseQuery
GO


-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_ApplicationDeleteTrigger
GO

-- Drop trigger of no longer existing table
DROP TRIGGER IWA_EnvironmentDeleteTrigger
GO

-- Drop no longer existing table
DROP TABLE IWA_Environment
GO

-- Dropped columns of table IWA_Case
ALTER TABLE IWA_Case DROP COLUMN EnvironmentId
GO


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_ApplicationDeleteTrigger
  ON IWA_Application FOR DELETE AS

  DELETE IWA_SignalEvent FROM IWA_SignalEvent, deleted
  WHERE IWA_SignalEvent.ApplicationId = deleted.ApplicationId

  DELETE IWA_Role FROM IWA_Role, deleted
  WHERE IWA_Role.ApplicationId = deleted.ApplicationId

  DELETE IWA_ApplicationProperty FROM IWA_ApplicationProperty, deleted
  WHERE IWA_ApplicationProperty.ApplicationId = deleted.ApplicationId

  DELETE IWA_User FROM IWA_User, deleted
  WHERE IWA_User.ApplicationId = deleted.ApplicationId

  DELETE IWA_ProcessModel FROM IWA_ProcessModel, deleted
  WHERE IWA_ProcessModel.ApplicationId = deleted.ApplicationId

  DELETE IWA_SecurityMember FROM IWA_SecurityMember, deleted
  WHERE IWA_SecurityMember.ApplicationId = deleted.ApplicationId

  DELETE IWA_SecurityDescriptor FROM IWA_SecurityDescriptor, deleted
  WHERE IWA_SecurityDescriptor.SecurityDescriptorId = deleted.SecurityDescriptorId


GO


-- Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_CaseQuery
(
  CaseId,
  BusinessCaseId,
  Kind,
  ApplicationId,
  ProcessModelId,
  TaskStartId,
  ActivatorId,
  CreatorId,
  CreatorUserName,
  CreatorUserDisplayName,
  CreatorTaskId,
  Environment,
  DisplayNameTemplate,
  Name,
  DisplayDescriptionTemplate,
  Description,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  "State",
  Priority,
  Stage,
  OwnerId,
  OwnerName,
  OwnerDisplayName,
  Category
)
AS
  SELECT
    IWA_Case.CaseId,
    IWA_Case.BusinessCaseId,
    IWA_Case.Kind,
    IWA_Case.ApplicationId,
    IWA_Case.ProcessModelId,
    IWA_Case.TaskStartId,
    IWA_Case.ActivatorId,
    IWA_Case.CreatorId,
    Creator.Name,
    Creator.DisplayName,
    IWA_Case.CreatorTaskId,
    IWA_Case.Environment,
    IWA_Case.DisplayNameTemplate,
    IWA_Case.Name,
    IWA_Case.DisplayDescriptionTemplate,
    IWA_Case.Description,
    IWA_Case.StartTimestamp,
    IWA_Case.EndTimestamp,
    IWA_Case.BusinessCalendar,
    IWA_Case.WorkingTime,
    IWA_Case.BusinessRuntime,
    IWA_Case."State",
    IWA_Case.Priority,
    IWA_Case.Stage,
    IWA_Case.OwnerId,
    Owner.MemberName,
    Owner.DisplayName,
    IWA_Case.Category
  FROM IWA_Case
    LEFT OUTER JOIN IWA_SecurityMember AS Creator ON IWA_Case.CreatorId = Creator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS Owner ON IWA_Case.OwnerId = Owner.SecurityMemberId
GO


-- Update Version
UPDATE IWA_Version SET Version=84
GO
