# SQL script to convert the database for MySQL from version 83 to version 84

# Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_CaseQuery;


# Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_ApplicationDeleteTrigger;
# Store Procedure to drop a foreign key constraint
CREATE PROCEDURE IWA_Drop_ForeignKey_Constraint(fk_schema VARCHAR(64), fk_table VARCHAR(64), fk_column VARCHAR(64))
BEGIN
  WHILE EXISTS(
    SELECT * FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
    WHERE REFERENCED_COLUMN_NAME IS NOT NULL
      AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
      AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
      AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
  ) 
  DO
    BEGIN
      SET @sqlstmt = (
        SELECT CONCAT('ALTER TABLE `',TABLE_SCHEMA,'`.',TABLE_NAME,' DROP FOREIGN KEY ',CONSTRAINT_NAME)
        FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
        WHERE REFERENCED_COLUMN_NAME IS NOT NULL
          AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
          AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
          AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
        LIMIT 1
      );
      PREPARE stmt1 FROM @sqlstmt;
      EXECUTE stmt1;
    END;
  END WHILE;
END;
;

CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_Case', 'EnvironmentId');

# Drop no longer existing table
DROP TABLE IWA_Environment;

# Dropped columns of table IWA_Case
ALTER TABLE IWA_Case DROP COLUMN EnvironmentId;


# Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_ApplicationDeleteTrigger
AFTER DELETE ON IWA_Application
FOR EACH ROW
BEGIN
  DELETE FROM IWA_SignalEvent
  WHERE IWA_SignalEvent.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_Role
  WHERE IWA_Role.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_ApplicationProperty
  WHERE IWA_ApplicationProperty.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_User
  WHERE IWA_User.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_ProcessModel
  WHERE IWA_ProcessModel.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_SecurityMember
  WHERE IWA_SecurityMember.ApplicationId = OLD.ApplicationId;

  DELETE FROM IWA_SecurityDescriptor
  WHERE IWA_SecurityDescriptor.SecurityDescriptorId = OLD.SecurityDescriptorId;

END
;


# Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_CaseQuery
(
  CaseId,
  BusinessCaseId,
  Kind,
  ApplicationId,
  ProcessModelId,
  TaskStartId,
  ActivatorId,
  CreatorId,
  CreatorUserName,
  CreatorUserDisplayName,
  CreatorTaskId,
  Environment,
  DisplayNameTemplate,
  Name,
  DisplayDescriptionTemplate,
  Description,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  `State`,
  Priority,
  Stage,
  OwnerId,
  OwnerName,
  OwnerDisplayName,
  Category
)
AS
  SELECT
    IWA_Case.CaseId,
    IWA_Case.BusinessCaseId,
    IWA_Case.Kind,
    IWA_Case.ApplicationId,
    IWA_Case.ProcessModelId,
    IWA_Case.TaskStartId,
    IWA_Case.ActivatorId,
    IWA_Case.CreatorId,
    Creator.Name,
    Creator.DisplayName,
    IWA_Case.CreatorTaskId,
    IWA_Case.Environment,
    IWA_Case.DisplayNameTemplate,
    IWA_Case.Name,
    IWA_Case.DisplayDescriptionTemplate,
    IWA_Case.Description,
    IWA_Case.StartTimestamp,
    IWA_Case.EndTimestamp,
    IWA_Case.BusinessCalendar,
    IWA_Case.WorkingTime,
    IWA_Case.BusinessRuntime,
    IWA_Case.`State`,
    IWA_Case.Priority,
    IWA_Case.Stage,
    IWA_Case.OwnerId,
    Owner.MemberName,
    Owner.DisplayName,
    IWA_Case.Category
  FROM IWA_Case
    LEFT OUTER JOIN IWA_SecurityMember AS Creator ON IWA_Case.CreatorId = Creator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS Owner ON IWA_Case.OwnerId = Owner.SecurityMemberId;

# Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_ForeignKey_Constraint;

# Update Version
UPDATE IWA_Version SET Version=84;
