-- SQL script to convert the database for Postgre SQL from version 83 to version 84

-- Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_CaseQuery;

ALTER TABLE IWA_Case DROP
 CONSTRAINT IWA_Case_EnvironmentId_fkey
;


-- Drop no longer existing table
DROP TABLE IWA_Environment;

-- Dropped columns of table IWA_Case
ALTER TABLE IWA_Case DROP COLUMN EnvironmentId;


-- Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_CaseQuery
(
  CaseId,
  BusinessCaseId,
  Kind,
  ApplicationId,
  ProcessModelId,
  TaskStartId,
  ActivatorId,
  CreatorId,
  CreatorUserName,
  CreatorUserDisplayName,
  CreatorTaskId,
  Environment,
  DisplayNameTemplate,
  Name,
  DisplayDescriptionTemplate,
  Description,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  "State",
  Priority,
  Stage,
  OwnerId,
  OwnerName,
  OwnerDisplayName,
  Category
)
AS
  SELECT
    IWA_Case.CaseId,
    IWA_Case.BusinessCaseId,
    IWA_Case.Kind,
    IWA_Case.ApplicationId,
    IWA_Case.ProcessModelId,
    IWA_Case.TaskStartId,
    IWA_Case.ActivatorId,
    IWA_Case.CreatorId,
    Creator.Name,
    Creator.DisplayName,
    IWA_Case.CreatorTaskId,
    IWA_Case.Environment,
    IWA_Case.DisplayNameTemplate,
    IWA_Case.Name,
    IWA_Case.DisplayDescriptionTemplate,
    IWA_Case.Description,
    IWA_Case.StartTimestamp,
    IWA_Case.EndTimestamp,
    IWA_Case.BusinessCalendar,
    IWA_Case.WorkingTime,
    IWA_Case.BusinessRuntime,
    IWA_Case."State",
    IWA_Case.Priority,
    IWA_Case.Stage,
    IWA_Case.OwnerId,
    Owner.MemberName,
    Owner.DisplayName,
    IWA_Case.Category
  FROM IWA_Case
    LEFT OUTER JOIN IWA_SecurityMember AS Creator ON IWA_Case.CreatorId = Creator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS Owner ON IWA_Case.OwnerId = Owner.SecurityMemberId;


-- Update Version
UPDATE IWA_Version SET Version=84;
