-- SQL script to convert the database for Microsoft SQL Server from version 85 to version 86


-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_ProcessModelVersionDeleteTrigger
GO

-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_UserDeleteTrigger
GO

-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_TaskDeleteTrigger
GO

-- Drop no longer existing table
DROP TABLE IWA_PageArchive
GO

-- Drop no longer existing table
DROP TABLE IWA_PageElement
GO

-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_ProcessModelVersionDeleteTrigger
  ON IWA_ProcessModelVersion FOR DELETE AS

  DELETE IWA_TaskElement FROM IWA_TaskElement, deleted
  WHERE IWA_TaskElement.ProcessModelVersionId = deleted.ProcessModelVersionId


GO


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_UserDeleteTrigger
  ON IWA_User FOR DELETE AS

  UPDATE IWA_Note  SET IWA_Note.UserId=NULL
  FROM IWA_Note, deleted
  WHERE IWA_Note.UserId = deleted.UserId

  UPDATE IWA_SecurityDescriptor  SET IWA_SecurityDescriptor.OwnerUserId=NULL
  FROM IWA_SecurityDescriptor, deleted
  WHERE IWA_SecurityDescriptor.OwnerUserId = deleted.UserId

  DELETE IWA_UserRole FROM IWA_UserRole, deleted
  WHERE IWA_UserRole.UserId = deleted.UserId

  DELETE IWA_UserAbsence FROM IWA_UserAbsence, deleted
  WHERE IWA_UserAbsence.UserId = deleted.UserId

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.UserId = deleted.UserId

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.SubstituteUserId = deleted.UserId

  DELETE IWA_UserProperty FROM IWA_UserProperty, deleted
  WHERE IWA_UserProperty.UserId = deleted.UserId

  DELETE IWA_AccessControl FROM IWA_AccessControl, deleted
  WHERE IWA_AccessControl.UserId = deleted.UserId

  DELETE IWA_UserLocation FROM IWA_UserLocation, deleted
  WHERE IWA_UserLocation.UserId = deleted.UserId


GO


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_TaskDeleteTrigger
  ON IWA_Task FOR DELETE AS

  UPDATE IWA_Case  SET IWA_Case.CreatorTaskId=NULL
  FROM IWA_Case, deleted
  WHERE IWA_Case.CreatorTaskId = deleted.TaskId

  UPDATE IWA_SignalEvent  SET IWA_SignalEvent.SentByTaskId=NULL
  FROM IWA_SignalEvent, deleted
  WHERE IWA_SignalEvent.SentByTaskId = deleted.TaskId

  DELETE IWA_IntermediateEvent FROM IWA_IntermediateEvent, deleted
  WHERE IWA_IntermediateEvent.TaskId = deleted.TaskId

  DELETE IWA_SignaledTask FROM IWA_SignaledTask, deleted
  WHERE IWA_SignaledTask.SignaledTaskId = deleted.TaskId

  DELETE IWA_TaskLocation FROM IWA_TaskLocation, deleted
  WHERE IWA_TaskLocation.TaskId = deleted.TaskId

  DELETE IWA_TaskNote FROM IWA_TaskNote, deleted
  WHERE IWA_TaskNote.TaskId = deleted.TaskId

  DELETE IWA_TaskSignalEventReceiver FROM IWA_TaskSignalEventReceiver, deleted
  WHERE IWA_TaskSignalEventReceiver.WaitingTaskId = deleted.TaskId

  DELETE IWA_TaskData FROM IWA_TaskData, deleted
  WHERE IWA_TaskData.TaskId = deleted.TaskId

  DELETE IWA_WorkflowEvent FROM IWA_WorkflowEvent, deleted
  WHERE IWA_WorkflowEvent.TaskId = deleted.TaskId


GO


-- Update Version
UPDATE IWA_Version SET Version=86
GO
