# SQL script to convert the database for MySQL from version 85 to version 86


# Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_ProcessModelVersionDeleteTrigger;

# Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_UserDeleteTrigger;

# Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_TaskDeleteTrigger;

# Drop no longer existing table
DROP TABLE IWA_PageArchive;

# Drop no longer existing table
DROP TABLE IWA_PageElement;

# Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_ProcessModelVersionDeleteTrigger
AFTER DELETE ON IWA_ProcessModelVersion
FOR EACH ROW
BEGIN
  DELETE FROM IWA_TaskElement
  WHERE IWA_TaskElement.ProcessModelVersionId = OLD.ProcessModelVersionId;

END
;


# Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_UserDeleteTrigger
AFTER DELETE ON IWA_User
FOR EACH ROW
BEGIN
  UPDATE IWA_Note
  SET UserId=NULL
  WHERE IWA_Note.UserId=OLD.UserId;

  UPDATE IWA_SecurityDescriptor
  SET OwnerUserId=NULL
  WHERE IWA_SecurityDescriptor.OwnerUserId=OLD.UserId;

  DELETE FROM IWA_UserRole
  WHERE IWA_UserRole.UserId = OLD.UserId;

  DELETE FROM IWA_UserAbsence
  WHERE IWA_UserAbsence.UserId = OLD.UserId;

  DELETE FROM IWA_UserSubstitute
  WHERE IWA_UserSubstitute.UserId = OLD.UserId;

  DELETE FROM IWA_UserSubstitute
  WHERE IWA_UserSubstitute.SubstituteUserId = OLD.UserId;

  DELETE FROM IWA_UserProperty
  WHERE IWA_UserProperty.UserId = OLD.UserId;

  DELETE FROM IWA_AccessControl
  WHERE IWA_AccessControl.UserId = OLD.UserId;

  DELETE FROM IWA_UserLocation
  WHERE IWA_UserLocation.UserId = OLD.UserId;

END
;


# Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_TaskDeleteTrigger
AFTER DELETE ON IWA_Task
FOR EACH ROW
BEGIN
  UPDATE IWA_SignalEvent
  SET SentByTaskId=NULL
  WHERE IWA_SignalEvent.SentByTaskId=OLD.TaskId;

  DELETE FROM IWA_IntermediateEvent
  WHERE IWA_IntermediateEvent.TaskId = OLD.TaskId;

  DELETE FROM IWA_SignaledTask
  WHERE IWA_SignaledTask.SignaledTaskId = OLD.TaskId;

  DELETE FROM IWA_TaskLocation
  WHERE IWA_TaskLocation.TaskId = OLD.TaskId;

  DELETE FROM IWA_TaskNote
  WHERE IWA_TaskNote.TaskId = OLD.TaskId;

  DELETE FROM IWA_TaskSignalEventReceiver
  WHERE IWA_TaskSignalEventReceiver.WaitingTaskId = OLD.TaskId;

  DELETE FROM IWA_TaskData
  WHERE IWA_TaskData.TaskId = OLD.TaskId;

  DELETE FROM IWA_WorkflowEvent
  WHERE IWA_WorkflowEvent.TaskId = OLD.TaskId;

END
;


# Update Version
UPDATE IWA_Version SET Version=86;
