-- SQL script to convert the database for Microsoft SQL Server from version 87 to version 88

-- Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_SignalEventQuery
GO

-- Drop unique constraint which depend on changed columns or was deleted
-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_User'
GO

-- Added columns of table IWA_User
ALTER TABLE IWA_User ADD
  Language VARCHAR(5) NULL
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_User ADD UNIQUE (ApplicationId, Name)
GO

-- Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_SignalEventQuery
(
  SignalEventId,
  ApplicationId,
  SignalCode,
  SentTimestamp,
  SentById,
  SentByUserName,
  SentByUserId,
  SentByTaskId,
  SentByProcessElementPid
)
AS
  SELECT
    IWA_SignalEvent.SignalEventId,
    IWA_SignalEvent.ApplicationId,
    IWA_SignalEvent.SignalCode,
    IWA_SignalEvent.SentTimestamp,
    IWA_SignalEvent.SentById,
    IWA_SecurityMember.Name,
    IWA_User.UserId,
    IWA_SignalEvent.SentByTaskId,
    IWA_SignalEvent.SentByProcessElementPid
  FROM IWA_SignalEvent
    LEFT OUTER JOIN IWA_SecurityMember ON IWA_SignalEvent.SentById = IWA_SecurityMember.SecurityMemberId
    LEFT OUTER JOIN IWA_User ON IWA_SecurityMember.SecurityMemberId = IWA_User.SecurityMemberId
GO

-- Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_Unique
GO

-- Update Version
UPDATE IWA_Version SET Version=88
GO
