-- SQL script to convert the database for Microsoft SQL Server from version 89 to version 90


-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_UserDeleteTrigger
GO

-- Drop unique constraint which depend on changed columns or was deleted
-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_RoleProperty'
GO

-- Create new unique constraint of table IWA_Role
-- Skipping generation of constraint 'UNIQUE IWA_Role_SecurityMemberIdUnique (SecurityMemberId)'. It was already re-generated in this script.
ALTER TABLE IWA_Role ADD UNIQUE (SecurityMemberId)
GO

-- Create new unique constraint of table IWA_User
-- Skipping generation of constraint 'UNIQUE IWA_User_SecurityMemberIdUnique (SecurityMemberId)'. It was already re-generated in this script.
ALTER TABLE IWA_User ADD UNIQUE (SecurityMemberId)
GO

-- Added columns of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty ADD
  SecurityMemberId VARCHAR(210) NULL
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_RoleProperty ADD UNIQUE (RoleId, PropertyName)
GO

-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_UserProperty'
GO

-- Added columns of table IWA_UserProperty
ALTER TABLE IWA_UserProperty ADD
  SecurityMemberId VARCHAR(210) NULL
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_UserProperty ADD UNIQUE (UserId, PropertyName)
GO

-- Added columns of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence ADD
  SecurityMemberId VARCHAR(210) NULL
GO

-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_UserSubstitute'
GO

-- Added columns of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute ADD
  SecurityMemberId VARCHAR(210) NULL
GO
ALTER TABLE IWA_UserSubstitute ADD
  SubstituteUserSecurityMemberId VARCHAR(210) NULL
GO
ALTER TABLE IWA_UserSubstitute ADD
  SubstituteRoleSecurityMemberId VARCHAR(210) NULL 
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_UserSubstitute ADD UNIQUE (UserId, SubstituteUserId, SubstituteForRoleId)
GO

-- Added columns of table IWA_Note
ALTER TABLE IWA_Note ADD
  SecurityMemberId VARCHAR(210) NULL
GO

-- Added columns of table IWA_UserLocation
ALTER TABLE IWA_UserLocation ADD
  SecurityMemberId VARCHAR(210) NULL
GO

-- Dropped columns of table IWA_SecurityDescriptor
ALTER TABLE IWA_SecurityDescriptor DROP COLUMN OwnerUserId
GO


-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_AccessControl'
GO

-- Added columns of table IWA_AccessControl
ALTER TABLE IWA_AccessControl ADD
  SecurityMemberId VARCHAR(210) NULL REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_AccessControl ADD UNIQUE (SecurityDescriptorId, PermissionId, RoleId, UserId)
GO

-- Create new indexes of table IWA_UserProperty
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserProperty') AND name='IWA_UserProperty_SecurityMemberIdIndex')
  CREATE INDEX IWA_UserProperty_SecurityMemberIdIndex ON IWA_UserProperty (SecurityMemberId)
GO

-- Create new indexes of table IWA_UserAbsence
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserAbsence') AND name='IWA_UserAbsence_SecurityMemberIdIndex')
  CREATE INDEX IWA_UserAbsence_SecurityMemberIdIndex ON IWA_UserAbsence (SecurityMemberId)
GO

-- Create new indexes of table IWA_UserSubstitute
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserSubstitute') AND name='IWA_UserSubstitute_SecurityMemberIdIndex')
  CREATE INDEX IWA_UserSubstitute_SecurityMemberIdIndex ON IWA_UserSubstitute (SecurityMemberId)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserSubstitute') AND name='IWA_UserSbs_SbsUserScrtMmbrIdIdx')
  CREATE INDEX IWA_UserSbs_SbsUserScrtMmbrIdIdx ON IWA_UserSubstitute (SubstituteUserSecurityMemberId)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserSubstitute') AND name='IWA_UserSbs_SbsRoleScrtMmbrIdIdx')
  CREATE INDEX IWA_UserSbs_SbsRoleScrtMmbrIdIdx ON IWA_UserSubstitute (SubstituteRoleSecurityMemberId)
GO

-- Create new indexes of table IWA_Note
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Note') AND name='IWA_Note_SecurityMemberIdIndex')
  CREATE INDEX IWA_Note_SecurityMemberIdIndex ON IWA_Note (SecurityMemberId)
GO

-- Create new indexes of table IWA_UserLocation
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserLocation') AND name='IWA_UserLocation_SecurityMemberId')
  CREATE INDEX IWA_UserLocation_SecurityMemberId ON IWA_UserLocation (SecurityMemberId)
GO

-- Create new indexes of table IWA_AccessControl
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_AccessControl') AND name='IWA_AccessControl_SecMembrIdIdx')
  CREATE INDEX IWA_AccessControl_SecMembrIdIdx ON IWA_AccessControl (SecurityMemberId)
GO

-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_UserDeleteTrigger
  ON IWA_User FOR DELETE AS

  UPDATE IWA_Note  SET IWA_Note.UserId=NULL
  FROM IWA_Note, deleted
  WHERE IWA_Note.UserId = deleted.UserId

  DELETE IWA_UserRole FROM IWA_UserRole, deleted
  WHERE IWA_UserRole.UserId = deleted.UserId

  DELETE IWA_UserAbsence FROM IWA_UserAbsence, deleted
  WHERE IWA_UserAbsence.UserId = deleted.UserId

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.UserId = deleted.UserId

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.SubstituteUserId = deleted.UserId

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.SubstituteUserSecurityMemberId = deleted.SecurityMemberId

  DELETE IWA_UserProperty FROM IWA_UserProperty, deleted
  WHERE IWA_UserProperty.UserId = deleted.UserId

  DELETE IWA_AccessControl FROM IWA_AccessControl, deleted
  WHERE IWA_AccessControl.UserId = deleted.UserId

  DELETE IWA_UserLocation FROM IWA_UserLocation, deleted
  WHERE IWA_UserLocation.UserId = deleted.UserId


GO

-- Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_Unique
GO

-- Drop removed indexes of table IWA_Role
DROP INDEX IWA_Role.IWA_Role_SecurityMemberIdIndex
GO

-- Drop removed indexes of table IWA_User
DROP INDEX IWA_User.IWA_User_SecurityMemberIdIndex
GO

-- Update Version
UPDATE IWA_Version SET Version=90
GO
