-- SQL script to convert the database for Oracle from version 89 to version 90


-- Drop removed indexes of table IWA_Role
DROP INDEX IWA_Role_SecurityMemberIdIndex;

-- Drop removed indexes of table IWA_User
DROP INDEX IWA_User_SecurityMemberIdIndex;

-- Create new unique constraint of table IWA_Role
ALTER TABLE IWA_Role ADD UNIQUE (SecurityMemberId);


-- Create new unique constraint of table IWA_User
ALTER TABLE IWA_User ADD UNIQUE (SecurityMemberId);

-- Added columns of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty ADD
  SecurityMemberId VARCHAR2(210) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE;

-- Added columns of table IWA_UserProperty
ALTER TABLE IWA_UserProperty ADD
  SecurityMemberId VARCHAR2(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;

-- Added columns of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence ADD
  SecurityMemberId VARCHAR2(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;

-- Added columns of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute ADD
  SecurityMemberId VARCHAR2(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserSubstitute ADD
  SubstituteUserSecurityMemberId VARCHAR2(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserSubstitute ADD
  SubstituteRoleSecurityMemberId VARCHAR2(210) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE;

-- Added columns of table IWA_Note
ALTER TABLE IWA_Note ADD
  SecurityMemberId VARCHAR2(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE SET NULL;

-- Added columns of table IWA_UserLocation
ALTER TABLE IWA_UserLocation ADD
  SecurityMemberId VARCHAR2(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;

-- Dropped columns of table IWA_SecurityDescriptor
ALTER TABLE IWA_SecurityDescriptor DROP COLUMN OwnerUserId;


-- Added columns of table IWA_AccessControl
ALTER TABLE IWA_AccessControl ADD
  SecurityMemberId VARCHAR2(210) REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE;

-- Create new indexes of table IWA_UserProperty
DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_UserProperty_SecurityMemberIdIndex ON IWA_UserProperty (SecurityMemberId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

-- Create new indexes of table IWA_UserAbsence
DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_UserAbsence_SecurityMemberIdIndex ON IWA_UserAbsence (SecurityMemberId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

-- Create new indexes of table IWA_UserSubstitute
DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_UserSubstitute_SecurityMemberIdIndex ON IWA_UserSubstitute (SecurityMemberId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_UserSbs_SbsUserScrtMmbrIdIdx ON IWA_UserSubstitute (SubstituteUserSecurityMemberId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_UserSbs_SbsRoleScrtMmbrIdIdx ON IWA_UserSubstitute (SubstituteRoleSecurityMemberId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

-- Create new indexes of table IWA_Note
DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_Note_SecurityMemberIdIndex ON IWA_Note (SecurityMemberId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

-- Create new indexes of table IWA_UserLocation
DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_UserLocation_SecurityMemberId ON IWA_UserLocation (SecurityMemberId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

-- Create new indexes of table IWA_AccessControl
DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_AccessControl_SecMembrIdIdx ON IWA_AccessControl (SecurityMemberId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;



-- Update Version
UPDATE IWA_Version SET Version=90;
