-- SQL script to convert the database for Postgre SQL from version 89 to version 90


-- Drop removed indexes of table IWA_Role
DROP INDEX IWA_Role_SecurityMemberIdIndex;

-- Drop removed indexes of table IWA_User
DROP INDEX IWA_User_SecurityMemberIdIndex;

-- Create new unique constraint of table IWA_Role
ALTER TABLE IWA_Role ADD UNIQUE (SecurityMemberId);

-- Create new unique constraint of table IWA_User
ALTER TABLE IWA_User ADD UNIQUE (SecurityMemberId);

-- Added columns of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty ADD COLUMN
  SecurityMemberId VARCHAR(210) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE;

-- Added columns of table IWA_UserProperty
ALTER TABLE IWA_UserProperty ADD COLUMN
  SecurityMemberId VARCHAR(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;

-- Added columns of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence ADD COLUMN
  SecurityMemberId VARCHAR(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;

-- Added columns of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute ADD COLUMN
  SecurityMemberId VARCHAR(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserSubstitute ADD COLUMN
  SubstituteUserSecurityMemberId VARCHAR(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserSubstitute ADD COLUMN
  SubstituteRoleSecurityMemberId VARCHAR(210) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE;

-- Added columns of table IWA_Note
ALTER TABLE IWA_Note ADD COLUMN
  SecurityMemberId VARCHAR(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE SET NULL;

-- Added columns of table IWA_UserLocation
ALTER TABLE IWA_UserLocation ADD COLUMN
  SecurityMemberId VARCHAR(210) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;

-- Dropped columns of table IWA_SecurityDescriptor
ALTER TABLE IWA_SecurityDescriptor DROP COLUMN OwnerUserId;


-- Added columns of table IWA_AccessControl
ALTER TABLE IWA_AccessControl ADD COLUMN
  SecurityMemberId VARCHAR(210) REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE;

-- Create new indexes of table IWA_UserProperty
CREATE INDEX IF NOT EXISTS IWA_UserProperty_SecurityMemberIdIndex
ON IWA_UserProperty (SecurityMemberId varchar_pattern_ops);


-- Create new indexes of table IWA_UserAbsence
CREATE INDEX IF NOT EXISTS IWA_UserAbsence_SecurityMemberIdIndex
ON IWA_UserAbsence (SecurityMemberId varchar_pattern_ops);


-- Create new indexes of table IWA_UserSubstitute
CREATE INDEX IF NOT EXISTS IWA_UserSubstitute_SecurityMemberIdIndex
ON IWA_UserSubstitute (SecurityMemberId varchar_pattern_ops);

CREATE INDEX IF NOT EXISTS IWA_UserSbs_SbsUserScrtMmbrIdIdx
ON IWA_UserSubstitute (SubstituteUserSecurityMemberId varchar_pattern_ops);

CREATE INDEX IF NOT EXISTS IWA_UserSbs_SbsRoleScrtMmbrIdIdx
ON IWA_UserSubstitute (SubstituteRoleSecurityMemberId varchar_pattern_ops);


-- Create new indexes of table IWA_Note
CREATE INDEX IF NOT EXISTS IWA_Note_SecurityMemberIdIndex
ON IWA_Note (SecurityMemberId varchar_pattern_ops);


-- Create new indexes of table IWA_UserLocation
CREATE INDEX IF NOT EXISTS IWA_UserLocation_SecurityMemberId
ON IWA_UserLocation (SecurityMemberId varchar_pattern_ops);


-- Create new indexes of table IWA_AccessControl
CREATE INDEX IF NOT EXISTS IWA_AccessControl_SecMembrIdIdx
ON IWA_AccessControl (SecurityMemberId varchar_pattern_ops);


-- Update Version
UPDATE IWA_Version SET Version=90;
