-- SQL script to convert the database for Microsoft SQL Server from version 91 to version 92


-- Drop removed indexes of table IWA_UserAbsence
DROP INDEX IWA_UserAbsence.IWA_UserAbsence_UserIdIndex
GO

-- Drop removed indexes of table IWA_UserSubstitute
DROP INDEX IWA_UserSubstitute.IWA_UserSubstitute_UserIdIndex
GO
DROP INDEX IWA_UserSubstitute.IWA_UserSubstitute_SubstituteUserIdIndex
GO

-- Drop removed indexes of table IWA_Note
DROP INDEX IWA_Note.IWA_Note_UserIdIndex
GO

-- Drop removed indexes of table IWA_UserLocation
DROP INDEX IWA_UserLocation.IWA_UserLocation_UserId
GO

-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_RoleDeleteTrigger
GO

-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_UserDeleteTrigger
GO

-- Drop unique constraint which depend on changed columns or was deleted
-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_RoleProperty'
GO

-- Dropped columns of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty DROP COLUMN RoleId
GO


-- Changed columns of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty ALTER COLUMN SecurityMemberId VARCHAR(210) NOT NULL
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_RoleProperty ADD UNIQUE (SecurityMemberId, PropertyName)
GO


-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_UserProperty'
GO

-- Drop indexes which depend on changed columns
DROP INDEX IWA_UserProperty.IWA_UserProperty_SecurityMemberIdIndex
GO

-- Dropped columns of table IWA_UserProperty
ALTER TABLE IWA_UserProperty DROP COLUMN UserId
GO


-- Changed columns of table IWA_UserProperty
ALTER TABLE IWA_UserProperty ALTER COLUMN SecurityMemberId VARCHAR(210) NOT NULL
GO

-- Create index which depend on changed columns
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserProperty') AND name='IWA_UserProperty_SecurityMemberIdIndex')
  CREATE INDEX IWA_UserProperty_SecurityMemberIdIndex ON IWA_UserProperty (SecurityMemberId)
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_UserProperty ADD UNIQUE (SecurityMemberId, PropertyName)
GO


-- Drop indexes which depend on changed columns
DROP INDEX IWA_UserAbsence.IWA_UserAbsence_SecurityMemberIdIndex
GO

-- Dropped columns of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence DROP COLUMN UserId
GO


-- Changed columns of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence ALTER COLUMN SecurityMemberId VARCHAR(210) NOT NULL
GO

-- Create index which depend on changed columns
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserAbsence') AND name='IWA_UserAbsence_SecurityMemberIdIndex')
  CREATE INDEX IWA_UserAbsence_SecurityMemberIdIndex ON IWA_UserAbsence (SecurityMemberId)
GO

-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_UserSubstitute'
GO

-- Drop indexes which depend on changed columns
DROP INDEX IWA_UserSubstitute.IWA_UserSubstitute_SecurityMemberIdIndex
GO
DROP INDEX IWA_UserSubstitute.IWA_UserSbs_SbsUserScrtMmbrIdIdx
GO

-- Dropped columns of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute DROP COLUMN UserId
GO

ALTER TABLE IWA_UserSubstitute DROP COLUMN SubstituteUserId
GO

ALTER TABLE IWA_UserSubstitute DROP COLUMN SubstituteForRoleId
GO


-- Changed columns of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute ALTER COLUMN SecurityMemberId VARCHAR(210) NOT NULL
GO
ALTER TABLE IWA_UserSubstitute ALTER COLUMN SubstituteUserSecurityMemberId VARCHAR(210) NOT NULL
GO

-- Create index which depend on changed columns
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserSubstitute') AND name='IWA_UserSubstitute_SecurityMemberIdIndex')
  CREATE INDEX IWA_UserSubstitute_SecurityMemberIdIndex ON IWA_UserSubstitute (SecurityMemberId)
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserSubstitute') AND name='IWA_UserSbs_SbsUserScrtMmbrIdIdx')
  CREATE INDEX IWA_UserSbs_SbsUserScrtMmbrIdIdx ON IWA_UserSubstitute (SubstituteUserSecurityMemberId)
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_UserSubstitute ADD UNIQUE (SecurityMemberId, SubstituteUserSecurityMemberId, SubstituteRoleSecurityMemberId)
GO


-- Dropped columns of table IWA_Note
ALTER TABLE IWA_Note DROP COLUMN UserId
GO


-- Drop indexes which depend on changed columns
DROP INDEX IWA_UserLocation.IWA_UserLocation_SecurityMemberId
GO

-- Dropped columns of table IWA_UserLocation
ALTER TABLE IWA_UserLocation DROP COLUMN UserId
GO


-- Changed columns of table IWA_UserLocation
ALTER TABLE IWA_UserLocation ALTER COLUMN SecurityMemberId VARCHAR(210) NOT NULL
GO

-- Create index which depend on changed columns
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_UserLocation') AND name='IWA_UserLocation_SecurityMemberId')
  CREATE INDEX IWA_UserLocation_SecurityMemberId ON IWA_UserLocation (SecurityMemberId)
GO

-- Drop unique constraint which depend on changed columns or was deleted
EXECUTE IWA_Drop_Unique 'IWA_AccessControl'
GO

-- Drop indexes which depend on changed columns
DROP INDEX IWA_AccessControl.IWA_AccessControl_SecMembrIdIdx
GO

-- Dropped columns of table IWA_AccessControl
ALTER TABLE IWA_AccessControl DROP COLUMN RoleId
GO

ALTER TABLE IWA_AccessControl DROP COLUMN UserId
GO


-- Changed columns of table IWA_AccessControl
ALTER TABLE IWA_AccessControl ALTER COLUMN SecurityMemberId VARCHAR(210) NOT NULL
GO

-- Create index which depend on changed columns
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_AccessControl') AND name='IWA_AccessControl_SecMembrIdIdx')
  CREATE INDEX IWA_AccessControl_SecMembrIdIdx ON IWA_AccessControl (SecurityMemberId)
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_AccessControl ADD UNIQUE (SecurityDescriptorId, PermissionId, SecurityMemberId)
GO

-- Create new indexes of table IWA_RoleProperty
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_RoleProperty') AND name='IWA_RoleProperty_SecurityMemberIdIndex')
  CREATE INDEX IWA_RoleProperty_SecurityMemberIdIndex ON IWA_RoleProperty (SecurityMemberId)
GO

-- Create new unique constraint of table IWA_RoleProperty
-- Skipping generation of constraint 'UNIQUE RoleProperty_SecMemberIdPropertyNameIndex (SecurityMemberId, PropertyName)'. It was already re-generated in this script.


-- Create new unique constraint of table IWA_UserProperty
-- Skipping generation of constraint 'UNIQUE UserProperty_SecMemberIdPropertyNameIndex (SecurityMemberId, PropertyName)'. It was already re-generated in this script.


-- Create new unique constraint of table IWA_UserSubstitute
-- Skipping generation of constraint 'UNIQUE IWA_UserSubstitute_SecMemberId_SubstituteSecMemberUser_SubstituteSecMemberRoleIndex (SecurityMemberId, SubstituteUserSecurityMemberId, SubstituteRoleSecurityMemberId)'. It was already re-generated in this script.


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_RoleDeleteTrigger
  ON IWA_Role FOR DELETE AS
  IF (@@ROWCOUNT > 0)
  BEGIN

    DELETE IWA_UserRole FROM IWA_UserRole, deleted
    WHERE IWA_UserRole.RoleId = deleted.RoleId

    DELETE IWA_Role FROM IWA_Role, deleted
    WHERE IWA_Role.ParentRoleId = deleted.RoleId

    DELETE IWA_RoleRoleMember FROM IWA_RoleRoleMember, deleted
    WHERE IWA_RoleRoleMember.RoleId = deleted.RoleId

    DELETE IWA_RoleRoleMember FROM IWA_RoleRoleMember, deleted
    WHERE IWA_RoleRoleMember.RoleMemberId = deleted.RoleId

    END
GO


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_UserDeleteTrigger
  ON IWA_User FOR DELETE AS

  DELETE IWA_UserRole FROM IWA_UserRole, deleted
  WHERE IWA_UserRole.UserId = deleted.UserId

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.SubstituteUserSecurityMemberId = deleted.SecurityMemberId


GO

-- Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_Unique
GO

-- Update Version
UPDATE IWA_Version SET Version=92
GO
