# SQL script to convert the database for MySQL from version 91 to version 92


# Drop removed indexes of table IWA_UserAbsence
DROP INDEX IWA_UserAbsence_UserId
ON IWA_UserAbsence;

# Drop removed indexes of table IWA_UserSubstitute
DROP INDEX IWA_UserSubstitute_UserIdIndex
ON IWA_UserSubstitute;
DROP INDEX IWA_UserSubstitute_SubstituteUserIdIndex
ON IWA_UserSubstitute;

# Drop removed indexes of table IWA_Note
DROP INDEX IWA_Note_UserIdIndex
ON IWA_Note;

# Drop removed indexes of table IWA_UserLocation
DROP INDEX IWA_UserLocation_UserId
ON IWA_UserLocation;

# Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_RoleDeleteTrigger;

# Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_UserDeleteTrigger;

# Drop unique constraint which depend on changed columns or was deleted
ALTER TABLE IWA_RoleProperty DROP INDEX RoleId;

# Dropped columns of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty DROP COLUMN RoleId;


# Changed columns of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty MODIFY
  SecurityMemberId VARCHAR(210) NOT NULL;


# Drop unique constraint which depend on changed columns or was deleted
ALTER TABLE IWA_UserProperty DROP INDEX UserId;

# Dropped columns of table IWA_UserProperty
ALTER TABLE IWA_UserProperty DROP COLUMN UserId;


# Changed columns of table IWA_UserProperty
ALTER TABLE IWA_UserProperty MODIFY
  SecurityMemberId VARCHAR(210) NOT NULL;


# Dropped columns of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence DROP COLUMN UserId;


# Changed columns of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence MODIFY
  SecurityMemberId VARCHAR(210) NOT NULL;


# Drop unique constraint which depend on changed columns or was deleted
ALTER TABLE IWA_UserSubstitute DROP INDEX UserId;

# Dropped columns of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute DROP COLUMN UserId;

ALTER TABLE IWA_UserSubstitute DROP COLUMN SubstituteUserId;

ALTER TABLE IWA_UserSubstitute DROP COLUMN SubstituteForRoleId;


# Changed columns of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute MODIFY
  SecurityMemberId VARCHAR(210) NOT NULL;
ALTER TABLE IWA_UserSubstitute MODIFY
  SubstituteUserSecurityMemberId VARCHAR(210) NOT NULL;


# Dropped columns of table IWA_Note
ALTER TABLE IWA_Note DROP COLUMN UserId;


# Dropped columns of table IWA_UserLocation
ALTER TABLE IWA_UserLocation DROP COLUMN UserId;


# Changed columns of table IWA_UserLocation
ALTER TABLE IWA_UserLocation MODIFY
  SecurityMemberId VARCHAR(210) NOT NULL;

# Dropped columns of table IWA_AccessControl
ALTER TABLE IWA_AccessControl DROP INDEX SecurityDescriptorId;
ALTER TABLE IWA_AccessControl DROP COLUMN RoleId;
ALTER TABLE IWA_AccessControl DROP COLUMN UserId;
ALTER TABLE IWA_AccessControl ADD UNIQUE (SecurityDescriptorId, PermissionId, SecurityMemberId);

# Changed columns of table IWA_AccessControl
ALTER TABLE IWA_AccessControl MODIFY
  SecurityMemberId VARCHAR(210) NOT NULL;


# Create new indexes of table IWA_RoleProperty
CREATE PROCEDURE create_index_if_not_exists(sqlstatement VARCHAR(256))
BEGIN
  DECLARE already_exists CONDITION FOR SQLSTATE '23000';
  DECLARE columns_indexed CONDITION FOR SQLSTATE '42000';

  DECLARE EXIT HANDLER FOR already_exists, columns_indexed
  BEGIN
    -- Ignore exceptions for index already exists or columns already indexed
  END;

  SET @sqlstmt = sqlstatement;
  PREPARE stmt FROM @sqlstmt;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
END;
;

CALL create_index_if_not_exists('CREATE INDEX IWA_RoleProperty_SecurityMemberIdIndex ON IWA_RoleProperty (SecurityMemberId);');
DROP PROCEDURE create_index_if_not_exists;

# Create new unique constraint of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty ADD UNIQUE (SecurityMemberId, PropertyName);


# Create new unique constraint of table IWA_UserProperty
ALTER TABLE IWA_UserProperty ADD UNIQUE (SecurityMemberId, PropertyName);


# Create new unique constraint of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute ADD UNIQUE (SecurityMemberId, SubstituteUserSecurityMemberId, SubstituteRoleSecurityMemberId);


# Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_RoleDeleteTrigger
AFTER DELETE ON IWA_Role
FOR EACH ROW
BEGIN
  DELETE FROM IWA_UserRole
  WHERE IWA_UserRole.RoleId = OLD.RoleId;

  DELETE FROM IWA_RoleRoleMember
  WHERE IWA_RoleRoleMember.RoleId = OLD.RoleId;

  DELETE FROM IWA_RoleRoleMember
  WHERE IWA_RoleRoleMember.RoleMemberId = OLD.RoleId;

END
;


# Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_UserDeleteTrigger
AFTER DELETE ON IWA_User
FOR EACH ROW
BEGIN
  DELETE FROM IWA_UserRole
  WHERE IWA_UserRole.UserId = OLD.UserId;

END
;

# Update Version
UPDATE IWA_Version SET Version=92;
