-- SQL script to convert the database for Microsoft SQL Server from version 95 to version 96

-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_UserDeleteTrigger
GO

-- Store Procedure to drop a primary key constraint
CREATE PROCEDURE IWA_Drop_PrimaryKey
@tableName varchar(100)
AS
DECLARE @pkName Varchar(255)
SET @pkName= (
  SELECT [name] FROM sysobjects
    WHERE [xtype] = 'PK'
          AND [parent_obj] = OBJECT_ID(N'[dbo].['+@tableName+N']')
)
DECLARE @dropSql varchar(4000)
SET @dropSql=
  'ALTER TABLE [dbo].['+@tableName+']
    DROP CONSTRAINT ['+@PkName+']'
EXEC(@dropSql)

GO

EXECUTE IWA_Drop_PrimaryKey @tableName='IWA_UserRole'
GO

DROP PROCEDURE IWA_Drop_PrimaryKey
GO

-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_RoleDeleteTrigger
GO

-- Added columns of table IWA_UserRole
ALTER TABLE IWA_UserRole ADD
  RoleSecurityMemberId VARCHAR(210)
GO
ALTER TABLE IWA_UserRole ADD
  UserSecurityMemberId VARCHAR(210)
GO

-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_RoleDeleteTrigger
  ON IWA_Role FOR DELETE AS
  IF (@@ROWCOUNT > 0)
  BEGIN

    DELETE IWA_Role FROM IWA_Role, deleted
    WHERE IWA_Role.ParentSecurityMemberId = deleted.SecurityMemberId

    DELETE IWA_RoleRoleMember FROM IWA_RoleRoleMember, deleted
    WHERE IWA_RoleRoleMember.RoleId = deleted.RoleId

    DELETE IWA_RoleRoleMember FROM IWA_RoleRoleMember, deleted
    WHERE IWA_RoleRoleMember.RoleMemberId = deleted.RoleId

    END
GO


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_UserDeleteTrigger
  ON IWA_User FOR DELETE AS

  DELETE IWA_UserSubstitute FROM IWA_UserSubstitute, deleted
  WHERE IWA_UserSubstitute.SubstituteUserSecurityMemberId = deleted.SecurityMemberId

GO

-- Find SecurityMemberId of UserId and set it
UPDATE IWA_UserRole SET
  RoleSecurityMemberId = (SELECT SecurityMemberId FROM IWA_Role t WHERE t.RoleId = IWA_UserRole.RoleId),
  UserSecurityMemberId = (SELECT SecurityMemberId FROM IWA_User t WHERE t.UserId = IWA_UserRole.UserId);
GO

-- Dropped columns of table IWA_UserRole
ALTER TABLE IWA_UserRole DROP COLUMN RoleId
GO

ALTER TABLE IWA_UserRole DROP COLUMN UserId
GO

ALTER TABLE IWA_UserRole ALTER COLUMN RoleSecurityMemberId VARCHAR(210) NOT NULL
GO
ALTER TABLE IWA_UserRole ALTER COLUMN UserSecurityMemberId VARCHAR(210) NOT NULL
GO

ALTER TABLE IWA_UserRole ADD PRIMARY KEY (RoleSecurityMemberId, UserSecurityMemberId)
GO



-- Update Version
UPDATE IWA_Version SET Version=96
GO
