-- SQL script to convert the database for Microsoft SQL Server from version 97 to version 98

-- Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_SignalEventQuery
GO

-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_RoleDeleteTrigger
GO

-- Store Procedure to drop a primary key constraint
CREATE PROCEDURE IWA_Drop_PrimaryKey
@tableName varchar(100)
AS
DECLARE @pkName Varchar(255)
SET @pkName= (
  SELECT [name] FROM sysobjects
    WHERE [xtype] = 'PK'
          AND [parent_obj] = OBJECT_ID(N'[dbo].['+@tableName+N']')
)
DECLARE @dropSql varchar(4000)
SET @dropSql=
  'ALTER TABLE [dbo].['+@tableName+']
    DROP CONSTRAINT ['+@PkName+']'
EXEC(@dropSql)

GO
  



CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

-- drop existing unique keys (SecurityMemberId)
EXECUTE IWA_Drop_Unique 'IWA_User'
GO

EXECUTE IWA_Drop_Unique 'IWA_Role'
GO


EXECUTE IWA_Drop_PrimaryKey @tableName='IWA_Role'
GO
ALTER TABLE IWA_Role ADD PRIMARY KEY (SecurityMemberId)
GO

 

EXECUTE IWA_Drop_PrimaryKey @tableName='IWA_User'
GO
ALTER TABLE IWA_User ADD PRIMARY KEY (SecurityMemberId)
GO
    

-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_RoleDeleteTrigger
  ON IWA_Role FOR DELETE AS
  IF (@@ROWCOUNT > 0)
  BEGIN

    DELETE IWA_Role FROM IWA_Role, deleted
    WHERE IWA_Role.ParentSecurityMemberId = deleted.SecurityMemberId

    DELETE IWA_RoleRoleMember FROM IWA_RoleRoleMember, deleted
    WHERE IWA_RoleRoleMember.RoleSecurityMemberId = deleted.SecurityMemberId

    DELETE IWA_RoleRoleMember FROM IWA_RoleRoleMember, deleted
    WHERE IWA_RoleRoleMember.RoleMemberSecurityMemberId = deleted.SecurityMemberId

    END
GO



-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_Role ADD UNIQUE (ApplicationId, Name)
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_User ADD UNIQUE (ApplicationId, Name)
GO 

 
DROP PROCEDURE IWA_Drop_PrimaryKey
GO

DROP PROCEDURE IWA_Drop_Unique
GO



-- redefine all foreign keys because they were not already created
ALTER TABLE IWA_RoleProperty ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_Role (SecurityMemberId) ON DELETE CASCADE
GO
ALTER TABLE IWA_UserProperty ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User (SecurityMemberId) ON DELETE CASCADE
GO
ALTER TABLE IWA_UserAbsence ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE
GO
ALTER TABLE IWA_UserSubstitute ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User (SecurityMemberId) ON DELETE CASCADE
GO
ALTER TABLE IWA_UserSubstitute ADD FOREIGN KEY (SubstituteRoleSecurityMemberId) REFERENCES IWA_Role (SecurityMemberId) ON DELETE CASCADE
GO
ALTER TABLE IWA_Note ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User (SecurityMemberId) ON DELETE SET NULL
GO
ALTER TABLE IWA_UserLocation ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE
GO
ALTER TABLE IWA_UserRole ADD FOREIGN KEY (RoleSecurityMemberId) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE
GO
ALTER TABLE IWA_UserRole ADD FOREIGN KEY (UserSecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE
GO


-- making ex primary keys unique
ALTER TABLE IWA_Role ADD UNIQUE (RoleId)
GO

ALTER TABLE IWA_User ADD UNIQUE (UserId)
GO

CREATE VIEW IWA_SignalEventQuery
(
  SignalEventId,
  ApplicationId,
  SignalCode,
  SentTimestamp,
  SentById,
  SentByUserName,
  SentByUserId,
  SentByTaskId,
  SentByProcessElementPid
)
AS
  SELECT
    IWA_SignalEvent.SignalEventId,
    IWA_SignalEvent.ApplicationId,
    IWA_SignalEvent.SignalCode,
    IWA_SignalEvent.SentTimestamp,
    IWA_SignalEvent.SentById,
    IWA_SecurityMember.Name,
    IWA_User.UserId,
    IWA_SignalEvent.SentByTaskId,
    IWA_SignalEvent.SentByProcessElementPid
  FROM IWA_SignalEvent
    LEFT OUTER JOIN IWA_SecurityMember ON IWA_SignalEvent.SentById = IWA_SecurityMember.SecurityMemberId
    LEFT OUTER JOIN IWA_User ON IWA_SecurityMember.SecurityMemberId = IWA_User.SecurityMemberId
GO

-- Update Version
UPDATE IWA_Version SET Version=98
GO
