-- SQL script to convert the database for Oracle from version 97 to version 98

DROP VIEW IWA_SignalEventQuery;

-- Drop unique constraint which depend on changed columns or was deleted
ALTER TABLE IWA_User DROP UNIQUE (SecurityMemberId) CASCADE;
ALTER TABLE IWA_Role DROP UNIQUE (SecurityMemberId) CASCADE;

-- drop primary keys
ALTER TABLE IWA_User DROP PRIMARY KEY;
ALTER TABLE IWA_Role DROP PRIMARY KEY;

-- make the ex-unique keys to primary keys
ALTER TABLE IWA_User ADD PRIMARY KEY (SecurityMemberId); 
ALTER TABLE IWA_Role ADD PRIMARY KEY (SecurityMemberId); 

-- redefine all foreign keys because they are dropped while dropping the unique keys
ALTER TABLE IWA_Note ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User (SecurityMemberId) ON DELETE SET NULL;
ALTER TABLE IWA_Role ADD FOREIGN KEY (ParentSecurityMemberId) REFERENCES IWA_Role (SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_RoleProperty ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_Role (SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserAbsence ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserLocation ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserProperty ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User (SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserSubstitute ADD FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User (SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserSubstitute ADD FOREIGN KEY (SubstituteRoleSecurityMemberId) REFERENCES IWA_Role (SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserSubstitute ADD FOREIGN KEY (SubstituteUserSecurityMemberId) REFERENCES IWA_User (SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_RoleRoleMember ADD FOREIGN KEY (RoleSecurityMemberId) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_RoleRoleMember ADD FOREIGN KEY (RoleMemberSecurityMemberId) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserRole ADD FOREIGN KEY (RoleSecurityMemberId) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE;
ALTER TABLE IWA_UserRole ADD FOREIGN KEY (UserSecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE;

-- making ex-primary keys unique
ALTER TABLE IWA_Role ADD UNIQUE (RoleId);
ALTER TABLE IWA_User ADD UNIQUE (UserId);

CREATE VIEW IWA_SignalEventQuery
(
  SignalEventId,
  ApplicationId,
  SignalCode,
  SentTimestamp,
  SentById,
  SentByUserName,
  SentByUserId,
  SentByTaskId,
  SentByProcessElementPid
)
AS
  SELECT
    IWA_SignalEvent.SignalEventId,
    IWA_SignalEvent.ApplicationId,
    IWA_SignalEvent.SignalCode,
    IWA_SignalEvent.SentTimestamp,
    IWA_SignalEvent.SentById,
    IWA_SecurityMember.Name,
    IWA_User.UserId,
    IWA_SignalEvent.SentByTaskId,
    IWA_SignalEvent.SentByProcessElementPid
  FROM IWA_SignalEvent
    LEFT OUTER JOIN IWA_SecurityMember ON IWA_SignalEvent.SentById = IWA_SecurityMember.SecurityMemberId
    LEFT OUTER JOIN IWA_User ON IWA_SecurityMember.SecurityMemberId = IWA_User.SecurityMemberId;

UPDATE IWA_Version SET Version=98;

