-- migrate oracle from version 99 to version 100

-- drop views
DROP VIEW IWA_SignalEventQuery;
DROP VIEW IWA_TaskQuery;
DROP VIEW IWA_CaseQuery;

-- drop trigger
DROP TRIGGER IWA_ApplicationDeleteTrg;

-- add columns
ALTER TABLE IWA_SecurityMember ADD SecuritySystemId NUMBER(20) NULL REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_Role ADD SecuritySystemId NUMBER(20) NULL REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_User ADD SecuritySystemId NUMBER(20) NULL REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_Case ADD SecuritySystemId NUMBER(20) NULL REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_Task ADD SecuritySystemId NUMBER(20) NULL REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_WorkflowEvent ADD SecuritySystemId NUMBER(20) NULL REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_IntermediateEvent ADD SecuritySystemId NUMBER(20) NULL REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_SignalEvent ADD SecuritySystemId NUMBER(20) NULL REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_SecuritySystem ADD SecurityDescriptorId NUMBER(20) NULL REFERENCES IWA_SecurityDescriptor(SecurityDescriptorId);

-- nullable
ALTER TABLE IWA_Role MODIFY ApplicationId NULL;
ALTER TABLE IWA_SignalEvent MODIFY ApplicationId NULL;

-- drop unique
ALTER TABLE IWA_User DROP UNIQUE (ApplicationId, Name) DROP INDEX;

-- indexes
DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_Case_SecSystemIdIndex ON IWA_Case (SecuritySystemId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_Task_SecSystemIdIndex ON IWA_Task (SecuritySystemId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_WorkflowEvent_SecSystemIdIdx ON IWA_WorkflowEvent (SecuritySystemId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_IntermediateEvent_SecSystemId ON IWA_IntermediateEvent (SecuritySystemId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

-- create trigger
CREATE TRIGGER IWA_ApplicationDeleteTrg
AFTER DELETE ON IWA_Application
FOR EACH ROW
BEGIN
  DELETE IWA_Role
  WHERE IWA_Role.ApplicationId = :old.ApplicationId;

  DELETE IWA_Library
  WHERE IWA_Library.ApplicationId = :old.ApplicationId;

  DELETE IWA_ProcessModel
  WHERE IWA_ProcessModel.ApplicationId = :old.ApplicationId;

END;
;

-- update version
UPDATE IWA_Version SET Version=100;
