# migrate mysql from version 100 to version 101

# create security systems
# multiply the id with 10 because we have two reserved entryies (0=system, 1=default)
INSERT INTO IWA_SecuritySystem (SecuritySystemId, Name, SecurityDescriptorId)
SELECT ApplicationId * 10, Name, SecurityDescriptorId
FROM IWA_Application;

# fill up data
UPDATE IWA_SecurityMember SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_User SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_Role SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_Case SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_Task SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_WorkflowEvent SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_IntermediateEvent SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_SignalEvent SET SecuritySystemId = ApplicationId * 10;

# not null
ALTER TABLE IWA_SecurityMember MODIFY SecuritySystemId BIGINT NOT NULL;
ALTER TABLE IWA_User MODIFY SecuritySystemId BIGINT NOT NULL;
ALTER TABLE IWA_Role MODIFY SecuritySystemId BIGINT NOT NULL;
ALTER TABLE IWA_Case MODIFY SecuritySystemId BIGINT NOT NULL;
ALTER TABLE IWA_Task MODIFY SecuritySystemId BIGINT NOT NULL;
ALTER TABLE IWA_WorkflowEvent MODIFY SecuritySystemId BIGINT NOT NULL;
ALTER TABLE IWA_IntermediateEvent MODIFY SecuritySystemId BIGINT NOT NULL;
ALTER TABLE IWA_SignalEvent MODIFY SecuritySystemId BIGINT NOT NULL;
ALTER TABLE IWA_SecuritySystem MODIFY SecurityDescriptorId BIGINT NOT NULL;

# foreign keys
ALTER TABLE IWA_SecurityMember ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_Role ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_User ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_Case ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_Task ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_WorkflowEvent ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_IntermediateEvent ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_SignalEvent ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
ALTER TABLE IWA_SecuritySystem ADD FOREIGN KEY (SecurityDescriptorId) REFERENCES IWA_SecurityDescriptor(SecurityDescriptorId);

# unique keys
ALTER TABLE IWA_User ADD UNIQUE (SecuritySystemId, Name);
ALTER TABLE IWA_Role DROP INDEX ApplicationId;
ALTER TABLE IWA_Role ADD UNIQUE (SecuritySystemId, ApplicationId, Name);

# drop columns
ALTER TABLE IWA_SecurityMember DROP COLUMN ApplicationId;
ALTER TABLE IWA_Application DROP COLUMN SecurityDescriptorId;
ALTER TABLE IWA_User DROP INDEX ApplicationId;
ALTER TABLE IWA_User DROP COLUMN ApplicationId;

# add indexes
CREATE PROCEDURE create_index_if_not_exists(sqlstatement VARCHAR(256))
BEGIN
  DECLARE already_exists CONDITION FOR SQLSTATE '23000';
  DECLARE columns_indexed CONDITION FOR SQLSTATE '42000';
  
  DECLARE EXIT HANDLER FOR already_exists, columns_indexed
  BEGIN
    -- Ignore exceptions for index already exists or columns already indexed
  END;

  SET @sqlstmt = sqlstatement;
  PREPARE stmt FROM @sqlstmt;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
END;
;

ALTER TABLE IWA_User DROP INDEX IWA_User_ExternalIdIndex;
CALL create_index_if_not_exists('CREATE INDEX IWA_User_ExternalIdIndex ON IWA_User (SecuritySystemId, ExternalId)');

ALTER TABLE IWA_User DROP INDEX IWA_User_ExternalSecurityNameIndex;
CALL create_index_if_not_exists('CREATE INDEX IWA_User_ExternalSecurityNameIndex ON IWA_User (SecuritySystemId, ExternalSecurityName(253))');

DROP PROCEDURE create_index_if_not_exists;

# update version
UPDATE IWA_Version SET Version=101;
