-- migrate postgresql from version 100 to version 101

-- create security systems
-- multiply the id with 10 because we have two reserved entries (0=system, 1=default)
INSERT INTO IWA_SecuritySystem (SecuritySystemId, Name, SecurityDescriptorId)
SELECT ApplicationId * 10, Name, SecurityDescriptorId
FROM IWA_Application;

-- fill up data
UPDATE IWA_SecurityMember SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_User SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_Role SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_Case SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_Task SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_WorkflowEvent SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_IntermediateEvent SET SecuritySystemId = ApplicationId * 10;
UPDATE IWA_SignalEvent SET SecuritySystemId = ApplicationId * 10;

-- not null
ALTER TABLE IWA_SecurityMember ALTER COLUMN SecuritySystemId SET NOT NULL;
ALTER TABLE IWA_User ALTER COLUMN SecuritySystemId SET NOT NULL;
ALTER TABLE IWA_Role ALTER COLUMN SecuritySystemId SET NOT NULL;
ALTER TABLE IWA_Case ALTER COLUMN SecuritySystemId SET NOT NULL;
ALTER TABLE IWA_Task ALTER COLUMN SecuritySystemId SET NOT NULL;
ALTER TABLE IWA_WorkflowEvent ALTER COLUMN SecuritySystemId SET NOT NULL;
ALTER TABLE IWA_IntermediateEvent ALTER COLUMN SecuritySystemId SET NOT NULL;
ALTER TABLE IWA_SignalEvent ALTER COLUMN SecuritySystemId SET NOT NULL;
ALTER TABLE IWA_SecuritySystem ALTER COLUMN SecurityDescriptorId SET NOT NULL;

-- unique keys
ALTER TABLE IWA_Role DROP CONSTRAINT IWA_Role_ApplicationId_Name_Key CASCADE;
ALTER TABLE IWA_Role ADD UNIQUE (SecuritySystemId, ApplicationId, Name);
ALTER TABLE IWA_User ADD UNIQUE (SecuritySystemId, Name);


-- drop columns
ALTER TABLE IWA_SecurityMember DROP COLUMN ApplicationId;
ALTER TABLE IWA_Application DROP COLUMN SecurityDescriptorId;
ALTER TABLE IWA_User DROP COLUMN ApplicationId;

-- drop triggers
DROP TRIGGER IWA_ApplicationDeleteTrigger ON IWA_Application;

-- add indexes
CREATE INDEX IF NOT EXISTS IWA_User_ExternalIdIndex ON IWA_User (SecuritySystemId, ExternalId varchar_pattern_ops);
CREATE INDEX IF NOT EXISTS IWA_User_ExternalSecurityNameIndex ON IWA_User (SecuritySystemId, ExternalSecurityName varchar_pattern_ops);

-- update version
UPDATE IWA_Version SET Version=101;
