-- migrate mssql from version 101 to version 102

DROP TRIGGER IWA_ApplicationDeleteTrigger
GO

DECLARE @defname VARCHAR(100), @cmd VARCHAR(1000)
SET @defname = 
(SELECT name
FROM sysobjects so JOIN sysconstraints sc
ON so.id = sc.constid
WHERE object_name(so.parent_obj) = 'IWA_Role'
AND so.xtype = 'D'
AND sc.colid =
 (SELECT colid FROM syscolumns
 WHERE id = object_id('dbo.IWA_Role') AND
 name = 'ApplicationId'))
SET @cmd = 'ALTER TABLE IWA_Role DROP CONSTRAINT '
+ @defname
EXEC(@cmd)
GO

CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_Role'
GO
select null
GO


GO
DROP INDEX IWA_Role.IWA_Role_ExternalSecurityNameIndex 
GO
IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Role') AND name='IWA_Role_ExternalSecurityNameIndex')
  CREATE INDEX IWA_Role_ExternalSecurityNameIndex ON IWA_Role (SecuritySystemId, ExternalSecurityName)
GO


ALTER TABLE IWA_Role DROP COLUMN ApplicationId
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_Role ADD UNIQUE (SecuritySystemId, Name)
GO
ALTER TABLE IWA_Role ADD UNIQUE (RoleId)
GO


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_ApplicationDeleteTrigger
  ON IWA_Application FOR DELETE AS

  DELETE IWA_ProcessModel FROM IWA_ProcessModel, deleted
  WHERE IWA_ProcessModel.ApplicationId = deleted.ApplicationId


GO

DROP PROCEDURE IWA_Drop_Unique
GO

UPDATE IWA_Version SET Version=102
GO
