# migrate mysql from version 101 to version 102

DROP TRIGGER IWA_ApplicationDeleteTrigger;

CREATE PROCEDURE IWA_Drop_ForeignKey_Constraint(fk_schema VARCHAR(64), fk_table VARCHAR(64), fk_column VARCHAR(64))
BEGIN
  WHILE EXISTS(
    SELECT * FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
    WHERE REFERENCED_COLUMN_NAME IS NOT NULL
      AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
      AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
      AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
  ) 
  DO
    BEGIN
      SET @sqlstmt = (
        SELECT CONCAT('ALTER TABLE `',TABLE_SCHEMA,'`.',TABLE_NAME,' DROP FOREIGN KEY ',CONSTRAINT_NAME)
        FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
        WHERE REFERENCED_COLUMN_NAME IS NOT NULL
          AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
          AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
          AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
        LIMIT 1
      );
      PREPARE stmt1 FROM @sqlstmt;
      EXECUTE stmt1;
    END;
  END WHILE;
END;
;
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_Role', 'ApplicationId');
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_Role', 'SecuritySystemId');
DROP PROCEDURE IWA_Drop_ForeignKey_Constraint;


ALTER TABLE IWA_Role Drop INDEX IWA_Role_ExternalSecurityNameIndex;
ALTER TABLE IWA_Role Drop INDEX SecuritySystemId;
ALTER TABLE IWA_Role DROP COLUMN ApplicationId;

ALTER TABLE IWA_Role ADD UNIQUE (SecuritySystemId, Name);
ALTER TABLE IWA_Role ADD FOREIGN KEY (SecuritySystemId) REFERENCES IWA_SecuritySystem(SecuritySystemId);
CREATE PROCEDURE create_index_if_not_exists(sqlstatement VARCHAR(256))
BEGIN
  DECLARE already_exists CONDITION FOR SQLSTATE '23000';
  DECLARE columns_indexed CONDITION FOR SQLSTATE '42000';

  DECLARE EXIT HANDLER FOR already_exists, columns_indexed
  BEGIN
    -- Ignore exceptions for index already exists or columns already indexed
  END;

  SET @sqlstmt = sqlstatement;
  PREPARE stmt FROM @sqlstmt;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
END;
;

CALL create_index_if_not_exists('CREATE INDEX IWA_Role_ExternalSecurityNameIndex ON IWA_Role (SecuritySystemId, ExternalSecurityName(253));');

DROP PROCEDURE create_index_if_not_exists;


CREATE TRIGGER IWA_ApplicationDeleteTrigger
AFTER DELETE ON IWA_Application
FOR EACH ROW
BEGIN
  DELETE FROM IWA_ProcessModel
  WHERE IWA_ProcessModel.ApplicationId = OLD.ApplicationId;

END
;

UPDATE IWA_Version SET Version=102;
