-- SQL script to convert the database for Microsoft SQL Server from version 105 to version 106


-- Create new added tables
CREATE TABLE IWA_CaseLocalized
(
  CaseLocalizedId BIGINT NOT NULL,
  CaseId BIGINT NOT NULL REFERENCES IWA_Case(CaseId) ON DELETE CASCADE,
  LanguageId BIGINT NOT NULL,
  Name VARCHAR(200) NOT NULL,
  Description VARCHAR(MAX) NOT NULL,
  PRIMARY KEY (CaseLocalizedId),
  UNIQUE (CaseId, LanguageId)
)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_CaseLocalized') AND name='IWA_CaseLocalized_NameIndex')
  CREATE INDEX IWA_CaseLocalized_NameIndex ON IWA_CaseLocalized (Name)
GO

CREATE TABLE IWA_TaskLocalized
(
  TaskLocalizedId BIGINT NOT NULL,
  TaskId BIGINT NOT NULL REFERENCES IWA_Task(TaskId) ON DELETE CASCADE,
  LanguageId BIGINT NOT NULL,
  Name VARCHAR(200) NOT NULL,
  Description VARCHAR(MAX) NOT NULL,
  PRIMARY KEY (TaskLocalizedId),
  UNIQUE (TaskId, LanguageId)
)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_TaskLocalized') AND name='IWA_TaskLocalized_NameIndex')
  CREATE INDEX IWA_TaskLocalized_NameIndex ON IWA_TaskLocalized (Name)
GO

CREATE TABLE IWA_Language
(
  LanguageId BIGINT NOT NULL,
  SecuritySystemId BIGINT NOT NULL REFERENCES IWA_SecuritySystem(SecuritySystemId),
  IsDefault BIT NOT NULL,
  Locale VARCHAR(25) NOT NULL,
  PRIMARY KEY (LanguageId),
  UNIQUE (SecuritySystemId, Locale)
)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Language') AND name='IWA_Language_SecuritySystemId')
  CREATE INDEX IWA_Language_SecuritySystemId ON IWA_Language (SecuritySystemId)
GO

ALTER TABLE IWA_CaseLocalized ADD FOREIGN KEY (LanguageId) REFERENCES IWA_Language(LanguageId)
GO

ALTER TABLE IWA_TaskLocalized ADD FOREIGN KEY (LanguageId) REFERENCES IWA_Language(LanguageId)
GO


-- Update Version
UPDATE IWA_Version SET Version=106
GO
