-- SQL script to convert the database for Oracle from version 105 to version 106


-- Create new added tables
CREATE TABLE IWA_CaseLocalized
(
  CaseLocalizedId NUMBER(20) NOT NULL,
  CaseId NUMBER(20) NOT NULL REFERENCES IWA_Case(CaseId) ON DELETE CASCADE,
  LanguageId NUMBER(20) NOT NULL,
  Name VARCHAR2(200),
  Description CLOB NOT NULL,
  PRIMARY KEY (CaseLocalizedId),
  UNIQUE (CaseId, LanguageId)
)
TABLESPACE ${tablespaceName};

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_CaseLocalized_NameIndex ON IWA_CaseLocalized (Name) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

CREATE TABLE IWA_TaskLocalized
(
  TaskLocalizedId NUMBER(20) NOT NULL,
  TaskId NUMBER(20) NOT NULL REFERENCES IWA_Task(TaskId) ON DELETE CASCADE,
  LanguageId NUMBER(20) NOT NULL,
  Name VARCHAR2(200),
  Description CLOB NOT NULL,
  PRIMARY KEY (TaskLocalizedId),
  UNIQUE (TaskId, LanguageId)
)
TABLESPACE ${tablespaceName};

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_TaskLocalized_NameIndex ON IWA_TaskLocalized (Name) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

CREATE TABLE IWA_Language
(
  LanguageId NUMBER(20) NOT NULL,
  SecuritySystemId NUMBER(20) NOT NULL REFERENCES IWA_SecuritySystem(SecuritySystemId),
  IsDefault NUMBER(1) NOT NULL,
  Locale VARCHAR2(25) NOT NULL,
  PRIMARY KEY (LanguageId),
  UNIQUE (SecuritySystemId, Locale)
)
TABLESPACE ${tablespaceName};

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_Language_SecuritySystemId ON IWA_Language (SecuritySystemId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN 
  NULL;
END;
;

ALTER TABLE IWA_CaseLocalized ADD
(
 FOREIGN KEY (LanguageId) REFERENCES IWA_Language(LanguageId)
);

ALTER TABLE IWA_TaskLocalized ADD
(
 FOREIGN KEY (LanguageId) REFERENCES IWA_Language(LanguageId)
);


-- Update Version
UPDATE IWA_Version SET Version=106;
