-- SQL script to convert the database for Postgre SQL from version 105 to version 106


-- Create new added tables
CREATE TABLE IWA_CaseLocalized
(
  CaseLocalizedId BIGINT NOT NULL,
  CaseId BIGINT NOT NULL REFERENCES IWA_Case(CaseId) ON DELETE CASCADE,
  LanguageId BIGINT NOT NULL,
  Name VARCHAR(200) NOT NULL,
  Description TEXT NOT NULL,
  PRIMARY KEY (CaseLocalizedId),
  UNIQUE (CaseId, LanguageId)
);

CREATE INDEX IF NOT EXISTS IWA_CaseLocalized_NameIndex
ON IWA_CaseLocalized (Name varchar_pattern_ops);

CREATE TABLE IWA_TaskLocalized
(
  TaskLocalizedId BIGINT NOT NULL,
  TaskId BIGINT NOT NULL REFERENCES IWA_Task(TaskId) ON DELETE CASCADE,
  LanguageId BIGINT NOT NULL,
  Name VARCHAR(200) NOT NULL,
  Description TEXT NOT NULL,
  PRIMARY KEY (TaskLocalizedId),
  UNIQUE (TaskId, LanguageId)
);

CREATE INDEX IF NOT EXISTS IWA_TaskLocalized_NameIndex
ON IWA_TaskLocalized (Name varchar_pattern_ops);

CREATE TABLE IWA_Language
(
  LanguageId BIGINT NOT NULL,
  SecuritySystemId BIGINT NOT NULL REFERENCES IWA_SecuritySystem(SecuritySystemId),
  IsDefault DECIMAL(1) NOT NULL,
  Locale VARCHAR(25) NOT NULL,
  PRIMARY KEY (LanguageId),
  UNIQUE (SecuritySystemId, Locale)
);

CREATE INDEX IF NOT EXISTS IWA_Language_SecuritySystemId
ON IWA_Language (SecuritySystemId);

ALTER TABLE IWA_CaseLocalized ADD FOREIGN KEY (LanguageId) REFERENCES IWA_Language(LanguageId);

ALTER TABLE IWA_TaskLocalized ADD FOREIGN KEY (LanguageId) REFERENCES IWA_Language(LanguageId);


-- Update Version
UPDATE IWA_Version SET Version=106;
