# SQL script to convert the database for MySQL from version 106 to version 107

# Move Name and Description of Task and Cases to new language specific tables
INSERT INTO IWA_Language SELECT SecuritySystemId, SecuritySystemId, 1, 'en' FROM IWA_SecuritySystem;

INSERT INTO IWA_TaskLocalized SELECT TaskId, TaskId, SecuritySystemId, Name, Description FROM IWA_Task;
 
INSERT INTO IWA_CaseLocalized SELECT CaseId, CaseId, SecuritySystemId, Name, Description FROM IWA_Case;


# Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_TaskQuery;

DROP VIEW IWA_CaseQuery;


# Drop removed indexes of table IWA_Case
DROP INDEX IWA_Case_NameIndex
ON IWA_Case;

# Drop removed indexes of table IWA_Task
DROP INDEX IWA_Task_NameIndex
ON IWA_Task;

# Dropped columns of table IWA_Case
ALTER TABLE IWA_Case DROP COLUMN DisplayNameTemplate;

ALTER TABLE IWA_Case DROP COLUMN Name;

ALTER TABLE IWA_Case DROP COLUMN DisplayDescriptionTemplate;

ALTER TABLE IWA_Case DROP COLUMN Description;


# Dropped columns of table IWA_Task
ALTER TABLE IWA_Task DROP COLUMN DisplayNameTemplate;

ALTER TABLE IWA_Task DROP COLUMN Name;

ALTER TABLE IWA_Task DROP COLUMN DisplayDescriptionTemplate;

ALTER TABLE IWA_Task DROP COLUMN Description;


CREATE PROCEDURE create_index_if_not_exists(sqlstatement VARCHAR(256))
BEGIN
  DECLARE already_exists CONDITION FOR SQLSTATE '23000';
  DECLARE columns_indexed CONDITION FOR SQLSTATE '42000';

  DECLARE EXIT HANDLER FOR already_exists, columns_indexed
  BEGIN
    -- Ignore exceptions for index already exists or columns already indexed
  END;

  SET @sqlstmt = sqlstatement;
  PREPARE stmt FROM @sqlstmt;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
END;
;

# Create new indexes of table IWA_Case
CALL create_index_if_not_exists('CREATE INDEX IWA_Case_StateCreatorIdIndex ON IWA_Case (`State`, CreatorId);');

# Create new indexes of table IWA_Task
CALL create_index_if_not_exists('CREATE INDEX IWA_Task_StateActivatorIdIndex ON IWA_Task (`State`, ActivatorId);');

CALL create_index_if_not_exists('CREATE INDEX IWA_Task_StateWorkerIdIndex ON IWA_Task (`State`, WorkerId);');

DROP PROCEDURE create_index_if_not_exists;


# Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_TaskQuery
(
  TaskId,
  CaseId,
  BusinessCaseId,
  ProcessModelId,
  SecuritySystemId,
  ApplicationId,
  StartTaskSwitchEventId,
  EndTaskSwitchEventId,
  TaskStartId,
  TaskEndId,
  WorkerId,
  WorkerUserName,
  WorkerUserDisplayName,
  WorkerSessionId,
  ActivatorId,
  OriginalActivatorId,
  OriginalActivatorName,
  OriginalActivatorDisplayName,
  ActivatorAvailable,
  ExpiryActivatorId,
  ExpiryActivatorName,
  ExpiryActivatorDisplayName,
  ExpiryPriority,
  ExpiryTimestamp,
  ExpiryTaskStartElementPid,
  IsExpired,
  ExpiredCreatorTaskId,
  TimeoutedCreatorIntrmdtEventId,
  DelayTimestamp,
  `State`,
  RequestPath,
  Name,
  Description,
  Priority,
  OriginalPriority,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  FailedTimeoutTimestamp,
  NumberOfFailures,
  NumberOfResumes,
  Category,
  IsUpdatedOnStart,
  IsOffline,
  ActivatorName,
  ActivatorDisplayName,
  LanguageId
)
AS
  SELECT
    IWA_Task.TaskId,
    IWA_Task.CaseId,
    IWA_Task.BusinessCaseId,
    IWA_Task.ProcessModelId,
    IWA_Task.SecuritySystemId,
    IWA_Task.ApplicationId,
    IWA_Task.StartTaskSwitchEventId,
    IWA_Task.EndTaskSwitchEventId,
    IWA_Task.TaskStartId,
    IWA_Task.TaskEndId,
    IWA_Task.WorkerId,
    Worker.Name,
    Worker.DisplayName,
    IWA_Task.WorkerSessionId,
    IWA_Task.ActivatorId,
    IWA_Task.OriginalActivatorId,
    OriginalActivator.MemberName,
    OriginalActivator.DisplayName,
    Activator.Enabled,
    IWA_Task.ExpiryActivatorId,
    ExpiryActivator.MemberName,
    ExpiryActivator.DisplayName,
    IWA_Task.ExpiryPriority,
    IWA_Task.ExpiryTimestamp,
    IWA_Task.ExpiryTaskStartElementPid,
    IWA_Task.IsExpired,
    IWA_Task.ExpiredCreatorTaskId,
    IWA_Task.TimeoutedCreatorIntrmdtEventId,
    IWA_Task.DelayTimestamp,
    IWA_Task.`State`,
    IWA_Task.RequestPath,
    TaskLocalized.Name,
    TaskLocalized.Description,
    IWA_Task.Priority,
    IWA_Task.OriginalPriority,
    IWA_Task.StartTimestamp,
    IWA_Task.EndTimestamp,
    IWA_Task.BusinessCalendar,
    IWA_Task.WorkingTime,
    IWA_Task.BusinessRuntime,
    IWA_Task.FailedTimeoutTimestamp,
    IWA_Task.NumberOfFailures,
    IWA_Task.NumberOfResumes,
    IWA_Task.Category,
    IWA_Task.IsUpdatedOnStart,
    IWA_Task.IsOffline,
    Activator.MemberName,
    Activator.DisplayName,
    TaskLocalized.LanguageId
  FROM IWA_Task
    LEFT OUTER JOIN IWA_SecurityMember AS Activator ON IWA_Task.ActivatorId = Activator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS OriginalActivator ON IWA_Task.OriginalActivatorId = OriginalActivator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS ExpiryActivator ON IWA_Task.ExpiryActivatorId = ExpiryActivator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS Worker ON IWA_Task.WorkerId = Worker.SecurityMemberId
    LEFT OUTER JOIN IWA_TaskLocalized AS TaskLocalized ON IWA_Task.TaskId = TaskLocalized.TaskId;

CREATE VIEW IWA_CaseQuery
(
  CaseId,
  BusinessCaseId,
  Kind,
  SecuritySystemId,
  ApplicationId,
  ProcessModelId,
  TaskStartId,
  ActivatorId,
  CreatorId,
  CreatorUserName,
  CreatorUserDisplayName,
  CreatorTaskId,
  Environment,
  Name,
  Description,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  `State`,
  Priority,
  Stage,
  OwnerId,
  OwnerName,
  OwnerDisplayName,
  Category,
  LanguageId
)
AS
  SELECT
    IWA_Case.CaseId,
    IWA_Case.BusinessCaseId,
    IWA_Case.Kind,
    IWA_Case.SecuritySystemId,
    IWA_Case.ApplicationId,
    IWA_Case.ProcessModelId,
    IWA_Case.TaskStartId,
    IWA_Case.ActivatorId,
    IWA_Case.CreatorId,
    Creator.Name,
    Creator.DisplayName,
    IWA_Case.CreatorTaskId,
    IWA_Case.Environment,
    CaseLocalized.Name,
    CaseLocalized.Description,
    IWA_Case.StartTimestamp,
    IWA_Case.EndTimestamp,
    IWA_Case.BusinessCalendar,
    IWA_Case.WorkingTime,
    IWA_Case.BusinessRuntime,
    IWA_Case.`State`,
    IWA_Case.Priority,
    IWA_Case.Stage,
    IWA_Case.OwnerId,
    Owner.MemberName,
    Owner.DisplayName,
    IWA_Case.Category,
    CaseLocalized.LanguageId
  FROM IWA_Case
    LEFT OUTER JOIN IWA_SecurityMember AS Creator ON IWA_Case.CreatorId = Creator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS Owner ON IWA_Case.OwnerId = Owner.SecurityMemberId
    LEFT OUTER JOIN IWA_CaseLocalized AS CaseLocalized ON IWA_Case.CaseId = CaseLocalized.CaseId;


# Update Version
UPDATE IWA_Version SET Version=107;
