-- SQL script to convert the database for Microsoft SQL Server from version 112 to version 113

ALTER TABLE IWA_SystemProperty
ALTER COLUMN PropertyValue VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_ProcessModelVersion
ALTER COLUMN Description VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_Library
ALTER COLUMN Description VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_CaseMap
ALTER COLUMN Model VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_RoleProperty
ALTER COLUMN PropertyValue VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_UserProperty
ALTER COLUMN PropertyValue VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_CaseLocalized
ALTER COLUMN Description VARCHAR(MAX) NOT NULL
GO

DROP INDEX IWA_TaskLocalized_NameIndex ON IWA_TaskLocalized
GO

ALTER TABLE IWA_TaskLocalized
ALTER COLUMN Name VARCHAR(200) NOT NULL
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_TaskLocalized') AND name='IWA_TaskLocalized_NameIndex')
  CREATE INDEX IWA_TaskLocalized_NameIndex ON IWA_TaskLocalized (Name)
GO


ALTER TABLE IWA_TaskLocalized
ALTER COLUMN Description VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_Note
ALTER COLUMN Note VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_TaskCustomTextField
ALTER COLUMN Value VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_CaseCustomTextField
ALTER COLUMN Value VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_BusinessData
ALTER COLUMN ObjectValue VARCHAR(MAX) NOT NULL
GO

ALTER TABLE IWA_AsyncProcessCaseData
ALTER COLUMN CaseData VARBINARY(MAX) NOT NULL
GO

ALTER TABLE IWA_TaskData
ALTER COLUMN TaskData VARBINARY(MAX) NOT NULL
GO

ALTER TABLE IWA_StartTaskData
ALTER COLUMN StartTaskData VARBINARY(MAX) NOT NULL
GO

ALTER TABLE IWA_IntermediateEventData
ALTER COLUMN ResultObjectData VARBINARY(MAX) NOT NULL
GO

ALTER TABLE IWA_SignalEventData
ALTER COLUMN SignalData VARBINARY(MAX) NOT NULL
GO


-- Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_CaseQuery
GO

DROP VIEW IWA_TaskQuery
GO

DROP VIEW IWA_CaseCustomField
GO

DROP VIEW IWA_TaskCustomField
GO


-- Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_CaseQuery
(
  CaseId,
  BusinessCaseId,
  Kind,
  SecuritySystemId,
  ApplicationId,
  ProcessModelId,
  TaskStartId,
  ActivatorId,
  CreatorId,
  CreatorUserName,
  CreatorUserDisplayName,
  CreatorTaskId,
  Environment,
  Name,
  Description,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  "State",
  Priority,
  Stage,
  OwnerId,
  OwnerName,
  OwnerDisplayName,
  Category,
  LanguageId
)
AS
  SELECT
    IWA_Case.CaseId,
    IWA_Case.BusinessCaseId,
    IWA_Case.Kind,
    IWA_Case.SecuritySystemId,
    IWA_Case.ApplicationId,
    IWA_Case.ProcessModelId,
    IWA_Case.TaskStartId,
    IWA_Case.ActivatorId,
    IWA_Case.CreatorId,
    Creator.Name,
    Creator.DisplayName,
    IWA_Case.CreatorTaskId,
    IWA_Case.Environment,
    CaseLocalized.Name,
    CaseLocalized.Description,
    IWA_Case.StartTimestamp,
    IWA_Case.EndTimestamp,
    IWA_Case.BusinessCalendar,
    IWA_Case.WorkingTime,
    IWA_Case.BusinessRuntime,
    IWA_Case."State",
    IWA_Case.Priority,
    IWA_Case.Stage,
    IWA_Case.OwnerId,
    Owner.MemberName,
    Owner.DisplayName,
    IWA_Case.Category,
    CaseLocalized.LanguageId
  FROM IWA_Case
    LEFT OUTER JOIN IWA_SecurityMember AS Creator ON IWA_Case.CreatorId = Creator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS Owner ON IWA_Case.OwnerId = Owner.SecurityMemberId
    LEFT OUTER JOIN IWA_CaseLocalized AS CaseLocalized ON IWA_Case.CaseId = CaseLocalized.CaseId
GO

CREATE VIEW IWA_TaskQuery
(
  TaskId,
  CaseId,
  BusinessCaseId,
  ProcessModelId,
  SecuritySystemId,
  ApplicationId,
  StartTaskSwitchEventId,
  EndTaskSwitchEventId,
  TaskStartId,
  TaskEndId,
  WorkerId,
  WorkerUserName,
  WorkerUserDisplayName,
  WorkerSessionId,
  ActivatorId,
  OriginalActivatorId,
  OriginalActivatorName,
  OriginalActivatorDisplayName,
  ActivatorAvailable,
  ExpiryActivatorId,
  ExpiryActivatorName,
  ExpiryActivatorDisplayName,
  ExpiryPriority,
  ExpiryTimestamp,
  ExpiryTaskStartElementPid,
  IsExpired,
  ExpiredCreatorTaskId,
  TimeoutedCreatorIntrmdtEventId,
  DelayTimestamp,
  "State",
  RequestPath,
  Name,
  Description,
  Priority,
  OriginalPriority,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendar,
  WorkingTime,
  BusinessRuntime,
  FailedTimeoutTimestamp,
  NumberOfFailures,
  NumberOfResumes,
  Category,
  IsUpdatedOnStart,
  IsOffline,
  ActivatorName,
  ActivatorDisplayName,
  LanguageId
)
AS
  SELECT
    IWA_Task.TaskId,
    IWA_Task.CaseId,
    IWA_Task.BusinessCaseId,
    IWA_Task.ProcessModelId,
    IWA_Task.SecuritySystemId,
    IWA_Task.ApplicationId,
    IWA_Task.StartTaskSwitchEventId,
    IWA_Task.EndTaskSwitchEventId,
    IWA_Task.TaskStartId,
    IWA_Task.TaskEndId,
    IWA_Task.WorkerId,
    Worker.Name,
    Worker.DisplayName,
    IWA_Task.WorkerSessionId,
    IWA_Task.ActivatorId,
    IWA_Task.OriginalActivatorId,
    OriginalActivator.MemberName,
    OriginalActivator.DisplayName,
    Activator.Enabled,
    IWA_Task.ExpiryActivatorId,
    ExpiryActivator.MemberName,
    ExpiryActivator.DisplayName,
    IWA_Task.ExpiryPriority,
    IWA_Task.ExpiryTimestamp,
    IWA_Task.ExpiryTaskStartElementPid,
    IWA_Task.IsExpired,
    IWA_Task.ExpiredCreatorTaskId,
    IWA_Task.TimeoutedCreatorIntrmdtEventId,
    IWA_Task.DelayTimestamp,
    IWA_Task."State",
    IWA_Task.RequestPath,
    TaskLocalized.Name,
    TaskLocalized.Description,
    IWA_Task.Priority,
    IWA_Task.OriginalPriority,
    IWA_Task.StartTimestamp,
    IWA_Task.EndTimestamp,
    IWA_Task.BusinessCalendar,
    IWA_Task.WorkingTime,
    IWA_Task.BusinessRuntime,
    IWA_Task.FailedTimeoutTimestamp,
    IWA_Task.NumberOfFailures,
    IWA_Task.NumberOfResumes,
    IWA_Task.Category,
    IWA_Task.IsUpdatedOnStart,
    IWA_Task.IsOffline,
    Activator.MemberName,
    Activator.DisplayName,
    TaskLocalized.LanguageId
  FROM IWA_Task
    LEFT OUTER JOIN IWA_SecurityMember AS Activator ON IWA_Task.ActivatorId = Activator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS OriginalActivator ON IWA_Task.OriginalActivatorId = OriginalActivator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS ExpiryActivator ON IWA_Task.ExpiryActivatorId = ExpiryActivator.SecurityMemberId
    LEFT OUTER JOIN IWA_SecurityMember AS Worker ON IWA_Task.WorkerId = Worker.SecurityMemberId
    LEFT OUTER JOIN IWA_TaskLocalized AS TaskLocalized ON IWA_Task.TaskId = TaskLocalized.TaskId
GO

CREATE VIEW IWA_CaseCustomField
(
  CaseId,
  Name,
  "Type",
  StringValue,
  TextValue,
  NumberValue,
  TimestampValue
)
AS
  SELECT
    StringField.CaseId,
    StringField.Name,
    'String',
    StringField."Value",
    NULL,
    NULL,
    NULL
  FROM IWA_CaseCustomStringField AS StringField
UNION ALL
  SELECT
    TextField.CaseId,
    TextField.Name,
    'Text',
    NULL,
    TextField."Value",
    NULL,
    NULL
  FROM IWA_CaseCustomTextField AS TextField
UNION ALL
  SELECT
    NumberField.CaseId,
    NumberField.Name,
    'Number',
    NULL,
    NULL,
    NumberField."Value",
    NULL
  FROM IWA_CaseCustomNumberField AS NumberField
UNION ALL
  SELECT
    TimestampField.CaseId,
    TimestampField.Name,
    'Timestamp',
    NULL,
    NULL,
    NULL,
    TimestampField."Value"
  FROM IWA_CaseCustomTimestampField AS TimestampField
GO

CREATE VIEW IWA_TaskCustomField
(
  TaskId,
  Name,
  "Type",
  StringValue,
  TextValue,
  NumberValue,
  TimestampValue
)
AS
  SELECT
    StringField.TaskId,
    StringField.Name,
    'String',
    StringField."Value",
    NULL,
    NULL,
    NULL
  FROM IWA_TaskCustomStringField AS StringField
UNION ALL
  SELECT
    TextField.TaskId,
    TextField.Name,
    'Text',
    NULL,
    TextField."Value",
    NULL,
    NULL
  FROM IWA_TaskCustomTextField AS TextField
UNION ALL
  SELECT
    NumberField.TaskId,
    NumberField.Name,
    'Number',
    NULL,
    NULL,
    NumberField."Value",
    NULL
  FROM IWA_TaskCustomNumberField AS NumberField
UNION ALL
  SELECT
    TimestampField.TaskId,
    TimestampField.Name,
    'Timestamp',
    NULL,
    NULL,
    NULL,
    TimestampField."Value"
  FROM IWA_TaskCustomTimestampField AS TimestampField
GO


-- Update Version
UPDATE IWA_Version SET Version=113
GO
