# migrate mysql from version 113 to version 114

# add missing foreign key
DELETE FROM IWA_AccessControl WHERE PermissionId NOT IN (SELECT PermissionId FROM IWA_Permission);
ALTER TABLE IWA_AccessControl ADD
(
 FOREIGN KEY (PermissionId) REFERENCES IWA_Permission(PermissionId) ON DELETE CASCADE
);

# add index for foreign keys
CREATE PROCEDURE create_index_if_not_exists(sqlstatement VARCHAR(256))
BEGIN
  DECLARE already_exists CONDITION FOR SQLSTATE '23000';
  DECLARE columns_indexed CONDITION FOR SQLSTATE '42000';

  DECLARE EXIT HANDLER FOR already_exists, columns_indexed
  BEGIN
    -- Ignore exceptions for index already exists or columns already indexed
  END;

  SET @sqlstmt = sqlstatement;
  PREPARE stmt FROM @sqlstmt;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
END;
;

CALL create_index_if_not_exists('CREATE INDEX IWA_AsyncProcessCaseData_CaseIdIndex ON IWA_AsyncProcessCaseData (CaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Application_SecuritySystemIdIndex ON IWA_Application (SecuritySystemId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_ProcessModel_ApplicationIdIndex ON IWA_ProcessModel (ApplicationId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_ProcessModelVersion_ProcessModelIdIndex ON IWA_ProcessModelVersion (ProcessModelId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Library_ApplicationIdIndex ON IWA_Library (ApplicationId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_LibrarySpecification_LibraryIdIndex ON IWA_LibrarySpecification (LibraryId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_LibraryVersionSpec_LibrarySpecificationIdIndex ON IWA_LibraryVersionSpec (LibrarySpecificationId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseMap_ProcessModelVersionIdIndex ON IWA_CaseMap (ProcessModelVersionId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseMap_ApplicationIdIndex ON IWA_CaseMap (ApplicationId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_SecurityMember_SecuritySystemIdIndex ON IWA_SecurityMember (SecuritySystemId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Role_SecuritySystemIdIndex ON IWA_Role (SecuritySystemId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_RoleRoleMember_RoleSecurityMemberIdIndex ON IWA_RoleRoleMember (RoleSecurityMemberId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_RoleRoleMember_RoleMemberSecurityMemberIdIndex ON IWA_RoleRoleMember (RoleMemberSecurityMemberId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_User_SecuritySystemIdIndex ON IWA_User (SecuritySystemId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_UserRole_RoleSecurityMemberIdIndex ON IWA_UserRole (RoleSecurityMemberId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_UserRole_UserSecurityMemberIdIndex ON IWA_UserRole (UserSecurityMemberId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskElement_ProcessModelVersionIdIndex ON IWA_TaskElement (ProcessModelVersionId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskStart_TaskElementIdIndex ON IWA_TaskStart (TaskElementId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskEnd_TaskElementIdIndex ON IWA_TaskEnd (TaskElementId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskSwitchEvent_TaskElementIdIndex ON IWA_TaskSwitchEvent (TaskElementId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskSwitchEvent_CaseIdIndex ON IWA_TaskSwitchEvent (CaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Case_ActivatorIdIndex ON IWA_Case (ActivatorId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseLocalized_CaseIdIndex ON IWA_CaseLocalized (CaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseLocalized_LanguageIdIndex ON IWA_CaseLocalized (LanguageId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Task_StartTaskSwitchEventIdIndex ON IWA_Task (StartTaskSwitchEventId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Task_EndTaskSwitchEventIdIndex ON IWA_Task (EndTaskSwitchEventId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Task_TaskStartIdIndex ON IWA_Task (TaskStartId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Task_TaskEndIdIndex ON IWA_Task (TaskEndId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Task_ExpiredCreatorTaskIdIndex ON IWA_Task (ExpiredCreatorTaskId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_Task_TimeoutedCreatorIntrmdtEventIdIndex ON IWA_Task (TimeoutedCreatorIntrmdtEventId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskLocalized_TaskIdIndex ON IWA_TaskLocalized (TaskId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskLocalized_LanguageIdIndex ON IWA_TaskLocalized (LanguageId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseMapEvent_ProcessCaseIdIndex ON IWA_CaseMapEvent (ProcessCaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskNote_TaskIdIndex ON IWA_TaskNote (TaskId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskNote_NoteIdIndex ON IWA_TaskNote (NoteId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseNote_CaseIdIndex ON IWA_CaseNote (CaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseNote_NoteIdIndex ON IWA_CaseNote (NoteId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskCustomStringField_TaskIdIndex ON IWA_TaskCustomStringField (TaskId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskCustomTextField_TaskIdIndex ON IWA_TaskCustomTextField (TaskId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskCustomNumberField_TaskIdIndex ON IWA_TaskCustomNumberField (TaskId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TaskCustomTimestampField_TaskIdIndex ON IWA_TaskCustomTimestampField (TaskId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseCstmStringFld_CaseIdIndex ON IWA_CaseCustomStringField (CaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseCstmTextFld_CaseIdIndex ON IWA_CaseCustomTextField (CaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseCstmNumberFld_CaseIdIndex ON IWA_CaseCustomNumberField (CaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_CaseCstmTmstmpFld_CaseIdIndex ON IWA_CaseCustomTimestampField (CaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_TskSigEvntRcvr_SgTskTskId ON IWA_TaskSignalEventReceiver (SignaledTaskTaskStartId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_SignalEvent_SecuritySystemId ON IWA_SignalEvent (SecuritySystemId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_SignalEvent_ApplicationId ON IWA_SignalEvent (ApplicationId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_SecurityDescriptor_SecurityDescriptorTypeId ON IWA_SecurityDescriptor (SecurityDescriptorTypeId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_SecuritySystem_SecurityDescriptorId ON IWA_SecuritySystem (SecurityDescriptorId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_PermissionGroup_ParentPermissionGroupId ON IWA_PermissionGroup (ParentPermissionGroupId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_SecurityDescriptorType_RootPermissionGroupId ON IWA_SecurityDescriptorType (RootPermissionGroupId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_AccessControl_PermissionIdIdx ON IWA_AccessControl (PermissionId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_PermissionGroupPermission_PermissionGroupId ON IWA_PermissionGroupPermission (PermissionGroupId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_PermissionGroupPermission_PermissionId ON IWA_PermissionGroupPermission (PermissionId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_BusinessCaseData_BusinessCaseId ON IWA_BusinessCaseData (BusinessCaseId);');
CALL create_index_if_not_exists('CREATE INDEX IWA_IntermediateEvent_ApplicationId ON IWA_IntermediateEvent (ApplicationId);');

DROP PROCEDURE create_index_if_not_exists;


# update version
UPDATE IWA_Version SET Version=114;
