-- SQL script to convert the database for Microsoft SQL Server from version 121 to version 122

-- Drop indexes

DROP INDEX IWA_Case_ApplicationIdCreationTimestampStateIndex ON IWA_Case
GO

DROP INDEX IWA_Case_StartTimestamp ON IWA_Case
GO

DROP INDEX IWA_Case_EndTimestamp ON IWA_Case
GO

DROP INDEX IWA_Task_StartTimestamp ON IWA_Task
GO

DROP INDEX IWA_Task_EndTimestamp ON IWA_Task
GO

DROP INDEX IWA_WorkflowEvent_EventTimestampIndex ON IWA_WorkflowEvent
GO

DROP INDEX IWA_CaseMapEvent_PrcRtryTstmp ON IWA_CaseMapEvent
GO

DROP INDEX IWA_IntrmdtEvnt_TimeoutTmstmp ON IWA_IntermediateEvent
GO

DROP INDEX IWA_TaskCstmTmstmpFldNmValIdx ON IWA_TaskCustomTimestampField
GO

DROP INDEX IWA_CaseCstmTmstmpFldNmValIdx ON IWA_CaseCustomTimestampField
GO

DROP INDEX IWA_SignalEvent_SentTimestamp ON IWA_SignalEvent
GO

DROP INDEX IWA_BusinessData_CrtdAtIdx ON IWA_BusinessData
GO

DROP INDEX IWA_BusinessData_ModAtIdx ON IWA_BusinessData
GO


-- Do changes

ALTER TABLE IWA_ProcessModelVersion ALTER COLUMN ReleaseTimestamp DATETIME2(3)
GO
ALTER TABLE IWA_ProcessModelVersion ALTER COLUMN ScheduledReleaseTimestamp DATETIME2(3)
GO
ALTER TABLE IWA_ProcessModelVersion ALTER COLUMN Changed DATETIME2(3) NOT NULL
GO

ALTER TABLE IWA_UserAbsence ALTER COLUMN StartTimestamp DATETIME2(3) NOT NULL
GO
ALTER TABLE IWA_UserAbsence ALTER COLUMN StopTimestamp DATETIME2(3)
GO

ALTER TABLE IWA_TaskSwitchEvent ALTER COLUMN ExecutionTimestamp DATETIME2(3) NOT NULL
GO

ALTER TABLE IWA_Case ALTER COLUMN StartTimestamp DATETIME2(3) NOT NULL
GO
ALTER TABLE IWA_Case ALTER COLUMN EndTimestamp DATETIME2(3)
GO
ALTER TABLE IWA_Case ALTER COLUMN ModifiedTimestamp DATETIME2(3) NOT NULL
GO

ALTER TABLE IWA_Task ALTER COLUMN ExpiryTimestamp DATETIME2(3)
GO
ALTER TABLE IWA_Task ALTER COLUMN DelayTimestamp DATETIME2(3)
GO
ALTER TABLE IWA_Task ALTER COLUMN StartTimestamp DATETIME2(3) NOT NULL
GO
ALTER TABLE IWA_Task ALTER COLUMN EndTimestamp DATETIME2(3)
GO
ALTER TABLE IWA_Task ALTER COLUMN ModifiedTimestamp DATETIME2(3) NOT NULL
GO
ALTER TABLE IWA_Task ALTER COLUMN FailedTimeoutTimestamp DATETIME2(3)
GO

ALTER TABLE IWA_WorkflowEvent ALTER COLUMN EventTimestamp DATETIME2(3) NOT NULL
GO

ALTER TABLE IWA_CaseMapEvent ALTER COLUMN "Timestamp" DATETIME2(3) NOT NULL
GO
ALTER TABLE IWA_CaseMapEvent ALTER COLUMN ProcessingRetryTimestamp DATETIME2(3)
GO

ALTER TABLE IWA_IntermediateEvent ALTER COLUMN EventTimestamp DATETIME2(3)
GO
ALTER TABLE IWA_IntermediateEvent ALTER COLUMN TimeoutTimestamp DATETIME2(3)
GO

ALTER TABLE IWA_Note ALTER COLUMN "Timestamp" DATETIME2(3) NOT NULL
GO

ALTER TABLE IWA_TaskCustomTimestampField ALTER COLUMN "Value" DATETIME2(3) NOT NULL
GO
ALTER TABLE IWA_CaseCustomTimestampField ALTER COLUMN "Value" DATETIME2(3) NOT NULL
GO

ALTER TABLE IWA_UserLocation ALTER COLUMN LocationTimestamp DATETIME2(3)
GO
ALTER TABLE IWA_TaskLocation ALTER COLUMN LocationTimestamp DATETIME2(3)
GO

ALTER TABLE IWA_SignalEvent ALTER COLUMN SentTimestamp DATETIME2(3) NOT NULL
GO

ALTER TABLE IWA_BusinessData ALTER COLUMN CreatedAt DATETIME2(3) NOT NULL
GO
ALTER TABLE IWA_BusinessData ALTER COLUMN ModifiedAt DATETIME2(3) NOT NULL
GO


-- Recreate indexes 

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Case') AND name='IWA_Case_ApplicationIdCreationTimestampStateIndex')
  CREATE INDEX IWA_Case_ApplicationIdCreationTimestampStateIndex ON IWA_Case (ApplicationId, StartTimestamp, "State")
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Case') AND name='IWA_Case_EndTimestamp')
  CREATE INDEX IWA_Case_EndTimestamp ON IWA_Case (EndTimestamp)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Case') AND name='IWA_Case_StartTimestamp')
  CREATE INDEX IWA_Case_StartTimestamp ON IWA_Case (StartTimestamp)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Task') AND name='IWA_Task_EndTimestamp')
  CREATE INDEX IWA_Task_EndTimestamp ON IWA_Task (EndTimestamp)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Task') AND name='IWA_Task_StartTimestamp')
  CREATE INDEX IWA_Task_StartTimestamp ON IWA_Task (StartTimestamp)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_WorkflowEvent') AND name='IWA_WorkflowEvent_EventTimestampIndex')
  CREATE INDEX IWA_WorkflowEvent_EventTimestampIndex ON IWA_WorkflowEvent (EventTimestamp)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_CaseMapEvent') AND name='IWA_CaseMapEvent_PrcRtryTstmp')
  CREATE INDEX IWA_CaseMapEvent_PrcRtryTstmp ON IWA_CaseMapEvent (ProcessingRetryTimestamp)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_IntermediateEvent') AND name='IWA_IntrmdtEvnt_TimeoutTmstmp')
  CREATE INDEX IWA_IntrmdtEvnt_TimeoutTmstmp ON IWA_IntermediateEvent (TimeoutTimestamp)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_TaskCustomTimestampField') AND name='IWA_TaskCstmTmstmpFldNmValIdx')
  CREATE INDEX IWA_TaskCstmTmstmpFldNmValIdx ON IWA_TaskCustomTimestampField (Name, "Value")
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_CaseCustomTimestampField') AND name='IWA_CaseCstmTmstmpFldNmValIdx')
  CREATE INDEX IWA_CaseCstmTmstmpFldNmValIdx ON IWA_CaseCustomTimestampField (Name, "Value")
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_SignalEvent') AND name='IWA_SignalEvent_SentTimestamp')
  CREATE INDEX IWA_SignalEvent_SentTimestamp ON IWA_SignalEvent (SentTimestamp)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_BusinessData') AND name='IWA_BusinessData_CrtdAtIdx')
  CREATE INDEX IWA_BusinessData_CrtdAtIdx ON IWA_BusinessData (CreatedAt)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_BusinessData') AND name='IWA_BusinessData_ModAtIdx')
  CREATE INDEX IWA_BusinessData_ModAtIdx ON IWA_BusinessData (ModifiedAt)
GO


-- Update Version
UPDATE IWA_Version SET Version=122
GO
