-- SQL script to convert the database for Oracle from version 130 to version 131


-- Create new added tables
CREATE TABLE IWA_Notification
(
  NotificationId VARCHAR2(36) NOT NULL,
  Kind VARCHAR2(100) NOT NULL,
  Payload CLOB NOT NULL,
  ReceiverId VARCHAR2(210) NOT NULL REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE,
  CreatedAt TIMESTAMP NOT NULL,
  PRIMARY KEY (NotificationId)
)
TABLESPACE ${tablespaceName};

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_Notification_KindIndex ON IWA_Notification (Kind) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_Notification_ReceiverIdIndex ON IWA_Notification (ReceiverId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_Notification_CreatedAtIndex ON IWA_Notification (CreatedAt) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN
  NULL;
END;
;

CREATE TABLE IWA_NotificationDelivery
(
  NotificationDeliveryId VARCHAR2(36) NOT NULL,
  NotificationId VARCHAR2(36) REFERENCES IWA_Notification(NotificationId) ON DELETE CASCADE,
  Channel VARCHAR2(100) NOT NULL,
  ReceiverId VARCHAR2(210) NOT NULL REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE,
  DeliveredAt TIMESTAMP,
  PRIMARY KEY (NotificationDeliveryId)
)
TABLESPACE ${tablespaceName};

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_NotificationDelivery_NotificationIdIndex ON IWA_NotificationDelivery (NotificationId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_NotificationDelivery_ChannelIndex ON IWA_NotificationDelivery (Channel) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_NotificationDelivery_ReceiverIdIndex ON IWA_NotificationDelivery (ReceiverId) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN
  NULL;
END;
;

DECLARE
  ALREADY_EXISTS EXCEPTION;
  COLUMNS_INDEXED EXCEPTION;
  PRAGMA EXCEPTION_INIT(ALREADY_EXISTS, -955);
  PRAGMA EXCEPTION_INIT(COLUMNS_INDEXED, -1408);
BEGIN
  EXECUTE IMMEDIATE 'CREATE INDEX IWA_NotificationDelivery_DeliveredAtIndex ON IWA_NotificationDelivery (DeliveredAt) TABLESPACE ${tablespaceName}';
EXCEPTION
  WHEN ALREADY_EXISTS OR COLUMNS_INDEXED THEN
  NULL;
END;
;

-- Update Version
UPDATE IWA_Version SET Version=131;
