# SQL script to convert the database for MySQL from version 137 to version 138

# Added columns of table IWA_Blob
ALTER TABLE IWA_Blob CHANGE FileId BlobId BIGINT NOT NULL;

ALTER TABLE IWA_Blob ADD COLUMN
  UUID VARCHAR(36);

# Create new indexes of table IWA_Blob
CREATE PROCEDURE create_index_if_not_exists(sqlstatement VARCHAR(256))
BEGIN
  DECLARE already_exists CONDITION FOR SQLSTATE '23000';
  DECLARE columns_indexed CONDITION FOR SQLSTATE '42000';

  DECLARE EXIT HANDLER FOR already_exists, columns_indexed
  BEGIN
    -- Ignore exceptions for index already exists or columns already indexed
  END;

  SET @sqlstmt = sqlstatement;
  PREPARE stmt FROM @sqlstmt;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
END;
;

CALL create_index_if_not_exists('CREATE INDEX IWA_Blob_FilePath ON IWA_Blob (FilePath(255));');

CALL create_index_if_not_exists('CREATE INDEX IWA_Blob_CreatedAt ON IWA_Blob (CreatedAt);');

CALL create_index_if_not_exists('CREATE INDEX IWA_Blob_FileName ON IWA_Blob (FileName);');

CALL create_index_if_not_exists('CREATE INDEX IWA_Blob_ModifiedAt ON IWA_Blob (ModifiedAt);');

CALL create_index_if_not_exists('CREATE INDEX IWA_Blob_SecuritySystemId ON IWA_Blob (SecuritySystemId);');

CALL create_index_if_not_exists('CREATE INDEX IWA_Blob_ApplicationId ON IWA_Blob (ApplicationId);');

DROP PROCEDURE create_index_if_not_exists;


# Create new unique constraint of table IWA_Blob
ALTER TABLE IWA_Blob ADD UNIQUE (UUID);


# Update Version
UPDATE IWA_Version SET Version=138;
