-- SQL script to convert the database for Microsoft SQL Server from version 139 to version 140


-- Drop unique constraint which depend on changed columns or was deleted
-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_Blob'
GO

-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_Blob ADD UNIQUE (BlobId)
GO

-- Store Procedure to drop a primary key constraint
CREATE PROCEDURE IWA_Drop_PrimaryKey
@tableName varchar(100)
AS
DECLARE @pkName Varchar(255)
SET @pkName= (
  SELECT [name] FROM sysobjects
    WHERE [xtype] = 'PK'
          AND [parent_obj] = OBJECT_ID(N'[dbo].['+@tableName+N']')
)
DECLARE @dropSql varchar(4000)
SET @dropSql=
  'ALTER TABLE [dbo].['+@tableName+']
    DROP CONSTRAINT ['+@PkName+']'
EXEC(@dropSql)

GO

EXECUTE IWA_Drop_PrimaryKey @tableName='IWA_Blob'
GO

DROP PROCEDURE IWA_Drop_PrimaryKey
GO

ALTER TABLE IWA_Blob ADD PRIMARY KEY (UUID)
GO

-- Create new unique constraint of table IWA_Blob
-- Skipping generation of constraint 'UNIQUE IWA_Blob_BlobId (BlobId)'. It was already re-generated in this script.

-- Drop temporary created stored procedures needed for conversion
DROP PROCEDURE IWA_Drop_Unique
GO

-- Update Version
UPDATE IWA_Version SET Version=140
GO
