-- SQL script to convert the database for Microsoft SQL Server from version 143 to version 144

CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_Blob'
GO

DROP INDEX IWA_Blob.IWA_Blob_ApplicationId
GO

ALTER TABLE IWA_Blob ALTER COLUMN ApplicationId BIGINT NOT NULL
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_Blob') AND name='IWA_Blob_ApplicationId')
  CREATE INDEX IWA_Blob_ApplicationId ON IWA_Blob (ApplicationId)
GO

ALTER TABLE IWA_Blob ADD UNIQUE (ApplicationId, FilePath)
GO
ALTER TABLE IWA_Blob ADD UNIQUE (BlobId)
GO
ALTER TABLE IWA_Blob ADD UNIQUE (SecuritySystemId, "Path")
GO

ALTER TABLE IWA_Blob ADD FOREIGN KEY (ApplicationId) REFERENCES IWA_Application (ApplicationId) ON DELETE CASCADE;

DROP PROCEDURE IWA_Drop_Unique
GO

UPDATE IWA_Version SET Version=144
GO
