-- Migrate Microsoft SQL Server from version 146 to version 147

-- Store Procedure to drop a unique constraint
CREATE PROCEDURE IWA_Drop_Unique
@tablename varchar(100)
AS
DECLARE @parentid integer
DECLARE @uqname varchar(100)
DECLARE table_cursor CURSOR FOR SELECT id FROM sysobjects WHERE name = @tablename
OPEN  table_cursor
FETCH NEXT FROM table_cursor INTO @parentid
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR ('TABLE %s not found in table sysobjects', 16, 1, @tablename)
  CLOSE table_cursor
  DEALLOCATE table_cursor
  RETURN
END
DECLARE uq_cursor CURSOR FOR
SELECT name from sysobjects WHERE parent_obj = @parentid AND xtype = 'UQ'
CLOSE table_cursor
DEALLOCATE table_cursor
OPEN uq_cursor
FETCH NEXT FROM uq_cursor INTO @uqname
IF @@FETCH_STATUS<>0
BEGIN
  RAISERROR('Unique constraint of table %s not found in table sysobjects', 16, 1, @tablename);
  CLOSE uq_cursor
  DEALLOCATE uq_cursor
  RETURN
END
WHILE @@FETCH_STATUS=0
BEGIN
  EXEC('ALTER TABLE '+@tablename+' DROP CONSTRAINT '+@uqname)
  FETCH NEXT FROM uq_cursor INTO @uqname
END
CLOSE uq_cursor
DEALLOCATE uq_cursor
GO

EXECUTE IWA_Drop_Unique 'IWA_Application'
GO
DECLARE @defname VARCHAR(100), @cmd VARCHAR(1000)
SET @defname = 
(SELECT name
FROM sysobjects so JOIN sysconstraints sc
ON so.id = sc.constid
WHERE object_name(so.parent_obj) = 'IWA_Application'
AND so.xtype = 'D'
AND sc.colid =
 (SELECT colid FROM syscolumns
 WHERE id = object_id('dbo.IWA_Application') AND
 name = 'Description'))
SET @cmd = 'ALTER TABLE IWA_Application DROP CONSTRAINT '
+ @defname
EXEC(@cmd)

GO
ALTER TABLE IWA_Application DROP COLUMN Description
GO
ALTER TABLE IWA_Application DROP COLUMN OwnerName
GO

ALTER TABLE IWA_Application ADD UNIQUE (Name)
GO
EXECUTE IWA_Drop_Unique 'IWA_ProcessModel'
GO
DECLARE @defname VARCHAR(100), @cmd VARCHAR(1000)
SET @defname = 
(SELECT name
FROM sysobjects so JOIN sysconstraints sc
ON so.id = sc.constid
WHERE object_name(so.parent_obj) = 'IWA_ProcessModel'
AND so.xtype = 'D'
AND sc.colid =
 (SELECT colid FROM syscolumns
 WHERE id = object_id('dbo.IWA_ProcessModel') AND
 name = 'Description'))
SET @cmd = 'ALTER TABLE IWA_ProcessModel DROP CONSTRAINT '
+ @defname
EXEC(@cmd)

GO
ALTER TABLE IWA_ProcessModel DROP COLUMN Description
GO

ALTER TABLE IWA_ProcessModel ADD UNIQUE (ApplicationId, Name)
GO
EXECUTE IWA_Drop_Unique 'IWA_ProcessModelVersion'
GO
ALTER TABLE IWA_ProcessModelVersion DROP COLUMN Description
GO
ALTER TABLE IWA_ProcessModelVersion DROP COLUMN ChangedBy
GO
ALTER TABLE IWA_ProcessModelVersion DROP COLUMN ChangedFromHost
GO
ALTER TABLE IWA_ProcessModelVersion DROP COLUMN NumberOfElements
GO

ALTER TABLE IWA_ProcessModelVersion ADD UNIQUE (ProcessModelId, VersionNumber)
GO
DROP PROCEDURE IWA_Drop_Unique
GO

UPDATE IWA_Version SET Version = 147
GO
