-- Migrate Microsoft SQL Server from version 153 to version 154

CREATE TABLE IWA_NotificationReceiver
(
  NotificationId VARCHAR(36) NOT NULL REFERENCES IWA_Notification(NotificationId) ON DELETE CASCADE,
  ReceiverId VARCHAR(210) NOT NULL,
  PRIMARY KEY (NotificationId, ReceiverId)
)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_NotificationReceiver') AND name='IWA_NotificationReceiver_NotificationIdIndex')
  CREATE INDEX IWA_NotificationReceiver_NotificationIdIndex ON IWA_NotificationReceiver (NotificationId)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('IWA_NotificationReceiver') AND name='IWA_NotificationReceiver_ReceiverIdIndex')
  CREATE INDEX IWA_NotificationReceiver_ReceiverIdIndex ON IWA_NotificationReceiver (ReceiverId)
GO

INSERT INTO IWA_NotificationReceiver (NotificationId, ReceiverId)
SELECT NotificationId, ReceiverId
FROM IWA_Notification
GO

DECLARE @defname VARCHAR(100), @cmd VARCHAR(1000)
SET @defname = 
(SELECT name
FROM sysobjects so JOIN sysconstraints sc
ON so.id = sc.constid
WHERE object_name(so.parent_obj) = 'IWA_Notification'
AND so.xtype = 'F'
AND sc.colid =
 (SELECT colid FROM syscolumns
 WHERE id = object_id('dbo.IWA_Notification') AND
 name = 'ReceiverId'))
SET @cmd = 'ALTER TABLE IWA_Notification DROP CONSTRAINT '
+ @defname
EXEC(@cmd)
GO

DROP INDEX IWA_Notification_ReceiverIdIndex ON IWA_Notification
GO

ALTER TABLE IWA_Notification DROP COLUMN ReceiverId
GO

ALTER TABLE IWA_NotificationReceiver ADD FOREIGN KEY (ReceiverId) REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE; 
GO

UPDATE IWA_Version SET Version = 154
GO
