# Migrate MySQL from version 153 to version 154

CREATE TABLE IWA_NotificationReceiver
(
  NotificationId VARCHAR(36) NOT NULL,
  ReceiverId VARCHAR(210) NOT NULL,
  PRIMARY KEY (NotificationId, ReceiverId),
  FOREIGN KEY (NotificationId) REFERENCES IWA_Notification(NotificationId) ON DELETE CASCADE,
  FOREIGN KEY (ReceiverId) REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE,
  INDEX IWA_NotificationReceiver_NotificationIdIndex (NotificationId),
  INDEX IWA_NotificationReceiver_ReceiverIdIndex (ReceiverId)
) ENGINE=InnoDB;

INSERT INTO IWA_NotificationReceiver (NotificationId, ReceiverId)
SELECT NotificationId, ReceiverId
FROM IWA_Notification;

CREATE PROCEDURE IWA_Drop_ForeignKey_Constraint(fk_schema VARCHAR(64), fk_table VARCHAR(64), fk_column VARCHAR(64))
BEGIN
  WHILE EXISTS(
    SELECT * FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
    WHERE REFERENCED_COLUMN_NAME IS NOT NULL
      AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
      AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
      AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
  ) 
  DO
    BEGIN
      SET @sqlstmt = (
        SELECT CONCAT('ALTER TABLE `',TABLE_SCHEMA,'`.',TABLE_NAME,' DROP FOREIGN KEY ',CONSTRAINT_NAME)
        FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
        WHERE REFERENCED_COLUMN_NAME IS NOT NULL
          AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
          AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
          AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
        LIMIT 1
      );
      PREPARE stmt1 FROM @sqlstmt;
      EXECUTE stmt1;
    END;
  END WHILE;
END;
;
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_Notification', 'ReceiverId');

DROP PROCEDURE IWA_Drop_ForeignKey_Constraint;

DROP INDEX IWA_Notification_ReceiverIdIndex
ON IWA_Notification;

ALTER TABLE IWA_Notification DROP COLUMN ReceiverId;

UPDATE IWA_Version SET Version = 154;
