-- Migrate Microsoft SQL Server from version 159 to version 160

CREATE PROCEDURE IWA_Drop_ForeignKey
@tableName varchar(100),
@columnName varchar(100)
AS
DECLARE @fkName Varchar(255)
SET @fkName= (
  SELECT fk.name
  FROM sys.foreign_keys fk
  INNER JOIN sys.foreign_key_columns fkc ON fkc.constraint_object_id = fk.object_id
  INNER JOIN sys.columns c1 ON fkc.parent_column_id = c1.column_id AND fkc.parent_object_id = c1.object_id
  INNER JOIN sys.columns c2 ON fkc.referenced_column_id = c2.column_id AND fkc.referenced_object_id = c2.object_id
  WHERE OBJECT_NAME(fk.parent_object_id)=@tableName AND c1.name=@columnName
)
DECLARE @dropSql varchar(4000)
SET @dropSql=
  'ALTER TABLE [dbo].['+@tableName+']
    DROP CONSTRAINT ['+@fkName+']'
EXEC(@dropSql)
GO

EXECUTE IWA_Drop_ForeignKey @tableName='IWA_Case', @columnName='OwnerId'
GO

DROP PROCEDURE IWA_Drop_ForeignKey
GO

DROP INDEX IWA_Case.IWA_Case_OwnerIdIndex
GO

CREATE TABLE IWA_CaseOwner
(
  CaseId BIGINT NOT NULL REFERENCES IWA_Case(CaseId) ON DELETE CASCADE,
  OwnerId VARCHAR(210) NOT NULL REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE,
  PRIMARY KEY (CaseId, OwnerId)
)
GO

INSERT INTO IWA_CaseOwner (CaseId, OwnerId)
SELECT CaseId, OwnerId FROM IWA_Case
WHERE OwnerId IS NOT NULL
GO

CREATE INDEX IWA_CaseOwner_CaseIdIndex ON IWA_CaseOwner (CaseId)
GO

CREATE INDEX IWA_CaseOwner_OwnerIdIndex ON IWA_CaseOwner (OwnerId)
GO

ALTER TABLE IWA_Case DROP COLUMN OwnerId
GO

UPDATE IWA_Version SET Version = 160
GO
