# Migrate MySQL from version 159 to version 160

CREATE PROCEDURE IWA_Drop_ForeignKey_Constraint(fk_schema VARCHAR(64), fk_table VARCHAR(64), fk_column VARCHAR(64))
BEGIN
  WHILE EXISTS(
    SELECT * FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
    WHERE REFERENCED_COLUMN_NAME IS NOT NULL
      AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
      AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
      AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
  ) 
  DO
    BEGIN
      SET @sqlstmt = (
        SELECT CONCAT('ALTER TABLE `',TABLE_SCHEMA,'`.',TABLE_NAME,' DROP FOREIGN KEY ',CONSTRAINT_NAME)
        FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
        WHERE REFERENCED_COLUMN_NAME IS NOT NULL
          AND CAST(TABLE_SCHEMA AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_schema AS CHAR CHARACTER SET ascii)
          AND CAST(TABLE_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_table AS CHAR CHARACTER SET ascii)
          AND CAST(COLUMN_NAME AS CHAR CHARACTER SET ascii) COLLATE ascii_general_ci = CAST(fk_column AS CHAR CHARACTER SET ascii)
        LIMIT 1
      );
      PREPARE stmt1 FROM @sqlstmt;
      EXECUTE stmt1;
    END;
  END WHILE;
END;
;
CALL IWA_Drop_ForeignKey_Constraint(SCHEMA(), 'IWA_Case', 'OwnerId');
DROP PROCEDURE IWA_Drop_ForeignKey_Constraint;

DROP INDEX IWA_Case_OwnerIdIndex
ON IWA_Case;

CREATE TABLE IWA_CaseOwner
(
  CaseId BIGINT NOT NULL,
  OwnerId VARCHAR(210) NOT NULL,
  PRIMARY KEY (CaseId, OwnerId),
  FOREIGN KEY (CaseId) REFERENCES IWA_Case(CaseId) ON DELETE CASCADE,
  FOREIGN KEY (OwnerId) REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE,
  INDEX IWA_CaseOwner_CaseIdIndex (CaseId),
  INDEX IWA_CaseOwner_OwnerIdIndex (OwnerId)
) ENGINE=InnoDB;

INSERT INTO IWA_CaseOwner (CaseId, OwnerId)
SELECT CaseId, OwnerId FROM IWA_Case
WHERE OwnerId IS NOT NULL; 

ALTER TABLE IWA_Case DROP COLUMN OwnerId;

UPDATE IWA_Version SET Version = 160;
